import pytest
import warnings
from insights.parsers import uname
from insights.tests import context_wrap

from distutils.version import LooseVersion, StrictVersion

import doctest


UNAME1 = "Linux foo.example.com 2.6.32-504.el6.x86_64 #1 SMP Tue Sep 16 01:56:35 EDT 2014 x86_64 x86_64 x86_64 GNU/Linux"
UNAME2 = "Linux rhel7box 3.10.0-229.el7.x86_64 #1 SMP Mon Mar 3 13:32:45 EST 2014 x86_64 x86_64 x86_64 GNU/Linux"
UNAME3 = "Linux map1a 2.6.18-53.el5PAE #1 SMP Wed Oct 10 16:48:18 EDT 2007 i686 i686 i386 GNU/Linux"
UNAME4 = "Linux cvlvtsmsrv01 3.10.0-229.el7.x86_64 #1 SMP Thu Jan 29 18:37:38 EST 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_ALT = "Linux cvlvtsmsrv01 4.11.0-44.el7.x86_64 #1 SMP Thu Jan 29 18:37:38 EST 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME4_DEBUG = "Linux cvlvtsmsrv01 3.10.0-229.el7.x86_64.debug #1 SMP Thu Jan 29 18:37:38 EST 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME5 = "Linux cvlvtsmsrv01 2.6.32-504.8.2.bgq.el6.x86_64 #1 SMP Thu Jan 29 18:37:38 EST 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_1 = "Linux localhost.localdomain 2.6.24.7-101.el5rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_1pre = "Linux localhost.localdomain 2.6.24.6-101.el5rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_1pre2 = "Linux localhost.localdomain 2.6.24-101.el5.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_1post = "Linux localhost.localdomain 2.6.24.7-101.1.el5rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_1post2 = "Linux localhost.localdomain 2.6.25-101.el5.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_2 = "Linux localhost.localdomain 2.6.33.9-rt31.66.el6rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_2pre = "Linux localhost.localdomain 2.6.33.9-rt31.65.el6rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_2pre2 = "Linux localhost.localdomain 2.6.33-65.el6.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_2post = "Linux localhost.localdomain 2.6.34.1-rt31.65.el6rt.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_2post2 = "Linux localhost.localdomain 2.6.34-65.el6.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_3 = "Linux localhost.localdomain 3.10.0-327.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_3pre = "Linux localhost.localdomain 3.10.0-326.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_3pre2 = "Linux localhost.localdomain 3.9.1-327.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_3post = "Linux localhost.localdomain 3.10.0-328.rt56.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_RT_3post2 = "Linux localhost.localdomain 3.10.1-327.204.el7.x86_64 #1 SMP PREEMPT RT Thu Oct 29 21:54:23 EDT 2015 x86_64 x86_64 x86_64 GNU/Linux"

UNAME_CVE_2016_0728_1 = 'Linux hostname.example.com 3.10.0-229.1.2.rt56.141.2.el7_1.x86_64 #1 SMP Fri Dec 19 12:09:25 EST 2014 x86_64 x86_64 x86_64 GNU/Linux'

UNAME_BLANK_LINE = """
Linux qqhrycsq2 2.6.32-279.el6.x86_64 #1 SMP Wed Jun 13 18:24:36 EDT 2012 x86_64 x86_64 x86_64 GNU/Linux

""".strip()

UNAME_FOREMAN_DEBUG = """
COMMAND> uname -a
Linux qqhrycsq2 2.6.32-279.el6.x86_64 #1 SMP Wed Jun 13 18:24:36 EDT 2012 x86_64 x86_64 x86_64 GNU/Linux
""".strip()

UNAME_ERROR_BLANK = ""
UNAME_ERROR_TOO_SHORT = "Linux localhost.localdomain"
UNAME_ERROR_TOO_SHORT2 = "Linux 2.6.32-279.el6.x86_64"
UNAME_ERROR_TOO_SHORT3 = "2.6.32-279.el6.x86_64"
UNAME_ERROR_ABBR_BAD_NVR = 'Linux bad-nvr 2'
UNAME_ERROR_TOO_MANY_REL_PARTS = 'Linux bad-parts 3.10.1.4.16-327.204.108.59.11.el7.x86_64'


def test_uname():
    uname1 = uname.Uname(context_wrap(UNAME1))
    uname2 = uname.Uname(context_wrap(UNAME2))
    uname3 = uname.Uname(context_wrap(UNAME3))
    uname4 = uname.Uname(context_wrap(UNAME4))
    uname4_debug = uname.Uname(context_wrap(UNAME4_DEBUG))
    uname_alt = uname.Uname(context_wrap(UNAME_ALT))
    uname5 = uname.Uname(context_wrap(UNAME5))
    uname6 = uname.Uname(context_wrap(UNAME_BLANK_LINE))
    uname7 = uname.Uname(context_wrap(UNAME_FOREMAN_DEBUG))

    # Test all the properties
    assert uname1.arch == 'x86_64'
    assert uname1.hw_platform == 'x86_64'
    assert uname1.kernel == '2.6.32-504.el6.x86_64'
    assert uname1.kernel_date == 'Tue Sep 16 01:56:35 EDT 2014'
    assert uname1.kernel_type == 'SMP'
    assert uname1.machine == 'x86_64'
    assert uname1.name == 'Linux'
    assert uname1.nodename == 'foo.example.com'
    assert uname1.os == 'GNU/Linux'
    assert uname1.processor == 'x86_64'
    assert uname1.redhat_release == uname.RedhatRelease(major=6, minor=6)
    assert uname1.release == '504.el6'
    assert uname1.release_arch == '504.el6.x86_64'
    assert uname1.release_tuple == (6, 6,)
    assert uname1.rhel_release == ['6', '6']
    assert uname1.ver_rel == '2.6.32-504.el6'
    assert uname1.version == '2.6.32'
    assert uname1.debug_kernel is False
    assert uname1._lv_release == LooseVersion('504.0.0.0.el6')
    assert uname1._lv_version == LooseVersion('2.6.32')
    assert uname1._rel_maj == '504'
    assert uname1._sv_version == StrictVersion('2.6.32')

    # Test that the debug kernel returns True
    assert uname4_debug.debug_kernel is True

    # Test kernel-alt pkgs
    assert uname_alt.release_tuple == (7, 4, "alt")

    # Test the equality and inequality operators
    assert uname1 != uname2
    assert uname2 == uname4
    assert uname2 == uname4_debug
    assert uname2 > uname1
    assert uname4 >= uname3
    assert uname4_debug >= uname3
    assert uname3 < uname2
    assert uname1 <= uname4
    assert uname1 <= uname4_debug

    # String and repr tests
    assert str(uname1) == 'version: 2.6.32; release: 504.el6; rel_maj: 504; lv_release: 504.0.0.0.el6'
    assert repr(uname1) == "<Uname 'version: 2.6.32; release: 504.el6; rel_maj: 504; lv_release: 504.0.0.0.el6'>"

    # Just a release
    uname_from_release = uname.Uname.from_release('7.2')
    assert uname_from_release.version == '3.10.0'
    assert uname_from_release.release == '327'

    # test obscure bits of parse_nvr()
    # Strict version value error
    nvr = uname.Uname.parse_nvr('quodge-327.204.el7.x86_64')
    assert nvr['version'] == 'quodge'
    assert nvr['_sv_version'] is None
    assert nvr['_lv_version'] == 'quodge'
    assert nvr['_lv_release'] == '327.204.0.0.el7'

    kernel1 = uname1  # 2.6.32-504.el6.x86_64
    assert [] == kernel1.fixed_by('2.6.32-220.1.el6', '2.6.32-504.el6')
    assert ['2.6.32-600.el6'] == kernel1.fixed_by('2.6.32-600.el6')
    assert [] == kernel1.fixed_by('2.6.32-600.el6', introduced_in='2.6.32-504.1.el6')
    assert ['2.6.33-100.el6'] == kernel1.fixed_by('2.6.33-100.el6')
    assert ['2.6.32-600.el6'] == kernel1.fixed_by('2.6.32-220.1.el6', '2.6.32-600.el6')
    assert ['2.6.32-504.1.el6'] == kernel1.fixed_by('2.6.32-504.1.el6')

    # test that 5 sections in a RH-released kernel name are not a problem
    kernel5 = uname5  # 2.6.32-504.8.2.bgq.el6.x86_64
    assert '2.6.32-504.8.2.bgq.el6' == kernel5.ver_rel

    assert uname6._sv_version == StrictVersion('2.6.32')
    assert uname6.arch == 'x86_64'
    assert uname6.hw_platform == 'x86_64'
    assert uname6.kernel == '2.6.32-279.el6.x86_64'
    assert uname6.kernel_date == 'Wed Jun 13 18:24:36 EDT 2012'

    assert uname7._sv_version == StrictVersion('2.6.32')
    assert uname7.arch == 'x86_64'
    assert uname7.hw_platform == 'x86_64'
    assert uname7.kernel == '2.6.32-279.el6.x86_64'
    assert uname7.kernel_date == 'Wed Jun 13 18:24:36 EDT 2012'

    # RT kernel tests
    uname_rt_1 = uname.Uname(context_wrap(UNAME_RT_1))
    assert uname_rt_1
    assert uname_rt_1.version == "2.6.24.7"
    assert uname_rt_1._sv_version is None
    assert uname_rt_1._lv_version == "2.6.24.7"
    assert uname_rt_1.release == "101.el5rt"
    assert uname_rt_1.arch == "x86_64"
    assert uname_rt_1 == UNAME_RT_1
    assert uname_rt_1 == uname.Uname.from_uname_str(UNAME_RT_1)
    assert uname_rt_1 > UNAME_RT_1pre
    assert uname_rt_1 > UNAME_RT_1pre2
    assert uname_rt_1 < UNAME_RT_1post
    assert uname_rt_1 < UNAME_RT_1post2
    assert uname_rt_1.rhel_release == ['-1', '-1']

    uname_rt_2 = uname.Uname(context_wrap(UNAME_RT_2))
    assert uname_rt_2
    assert uname_rt_2.version == "2.6.33.9"
    assert uname_rt_2._sv_version is None
    assert uname_rt_2._lv_version == "2.6.33.9"
    assert uname_rt_2.release == "rt31.66.el6rt"
    assert uname_rt_2.arch == "x86_64"
    assert uname_rt_2 == UNAME_RT_2
    assert uname_rt_2 == uname.Uname.from_uname_str(UNAME_RT_2)
    assert uname_rt_2 > UNAME_RT_2pre
    assert uname_rt_2 > UNAME_RT_2pre2
    assert uname_rt_2 < UNAME_RT_2post
    assert uname_rt_2 < UNAME_RT_2post2
    assert uname_rt_2.rhel_release == ['-1', '-1']

    uname_rt_3 = uname.Uname(context_wrap(UNAME_RT_3))
    assert uname_rt_3
    assert uname_rt_3.version == "3.10.0"
    assert uname_rt_3._sv_version == "3.10.0"
    assert uname_rt_3._lv_version == "3.10.0"
    assert uname_rt_3.release == "327.rt56.204.el7"
    assert uname_rt_3.arch == "x86_64"
    assert uname_rt_3 == UNAME_RT_3
    assert uname_rt_3 == uname.Uname.from_uname_str(UNAME_RT_3)
    assert uname_rt_3 > UNAME_RT_3pre
    assert uname_rt_3 > UNAME_RT_3pre2
    assert uname_rt_3 < UNAME_RT_3post
    assert uname_rt_3 < UNAME_RT_3post2
    assert uname_rt_3.rhel_release == ['7', '2']


def test_uname_errors():
    with pytest.raises(uname.UnameError) as e_info:
        uname.Uname(context_wrap(UNAME_ERROR_BLANK))
    assert 'Empty uname line' in str(e_info.value)
    with pytest.raises(uname.UnameError) as e_info:
        uname.Uname(context_wrap(UNAME_ERROR_TOO_SHORT))
    assert "Uname string appears invalid" in str(e_info.value)
    with pytest.raises(uname.UnameError) as e_info:
        uname.Uname(context_wrap(UNAME_ERROR_TOO_SHORT2))
    assert "Uname string appears invalid" in str(e_info.value)
    with pytest.raises(uname.UnameError) as e_info:
        uname.Uname(context_wrap(UNAME_ERROR_TOO_SHORT3))
    assert "Uname string appears invalid" in str(e_info.value)
    with pytest.raises(uname.UnameError) as e_info:
        uname.Uname(context_wrap(UNAME_ERROR_ABBR_BAD_NVR))
    assert "Too few parts in the uname version-release" in str(e_info.value)
    with pytest.raises(ValueError) as e_info:
        uname.Uname(context_wrap(UNAME_ERROR_TOO_MANY_REL_PARTS))
    assert "Too many sections encountered" in str(e_info.value)


def test_uname_from_rules():
    cve_2016_0728 = uname.Uname(context_wrap(UNAME_CVE_2016_0728_1))
    assert cve_2016_0728.kernel == '3.10.0-229.1.2.rt56.141.2.el7_1.x86_64'


def test_pad_release():
    assert "390.0.0.el6" == uname.pad_release("390.el6")
    assert "390.12.0.el6" == uname.pad_release("390.12.el6")
    assert "390.12.0.0.el6" == uname.pad_release("390.12.el6", 5)
    assert "390.0.0.0" == uname.pad_release("390")
    assert "390.12.0.0" == uname.pad_release("390.12")
    assert "1160.71.1.0" == uname.pad_release("1160.71.1")
    assert "390.12.0.0.rt5.3.el6" == uname.pad_release("390.12.rt5.3.el6", 7)
    assert "390.12.0.0.0.rt5.3" == uname.pad_release("390.12.rt5.3", 7)
    with pytest.raises(ValueError):
        uname.pad_release('390.11.12.13.el6')


def test_fixed_by():
    u = uname.Uname.from_uname_str("Linux qqhrycsq2 2.6.32-504.el6.x86_64 #1 SMP Wed Jun 13 18:24:36 EDT 2012 x86_64 x86_64 x86_64 GNU/Linux")
    assert [] == u.fixed_by('2.6.32-220.1.el6', '2.6.32-504.el6')
    assert ['2.6.32-600.el6'] == u.fixed_by('2.6.32-600.el6')
    assert [] == u.fixed_by('2.6.32-600.el6', introduced_in='2.6.32-504.1.el6')
    # Higher kernel version, lower release string should match
    assert ['2.6.33-100.el6'] == u.fixed_by('2.6.33-100.el6')
    assert ['2.6.32-600.el6'] == u.fixed_by('2.6.32-220.1.el6', '2.6.32-600.el6')


def test_fixed_by_warning():
    # For all remaining tests, cause the comparation warnings
    # to always be caught, and ignore other warnings.
    warning_msg = "Comparison of distribution part will be ingored"
    warnings.simplefilter("ignore")
    warnings.filterwarnings("always", message=warning_msg)
    with warnings.catch_warnings(record=True) as w:

        u = uname.Uname.from_uname_str("Linux qqhrycsq2 2.6.32-504.el6.x86_64 #1 SMP Wed Jun 13 18:24:36 EDT 2012 x86_64 x86_64 x86_64 GNU/Linux")
        # should be no comparation warning reported
        assert [] == u.fixed_by('2.6.32-220.1.el6', '2.6.32-504.el6')
        assert len(w) == 0

        # fixes without distribution part
        assert [] == u.fixed_by('2.6.32-504')
        assert len(w) == 1

        # fixes only has kernel version part
        assert [] == u.fixed_by('2.6.32-')
        assert len(w) == 2

        # introduce without distribution part
        assert ['2.6.32-600.el6'] == u.fixed_by('2.6.32-600.el6', introduced_in='2.6.32-504')
        assert len(w) == 3

        # introduce only has kernel version part
        assert ['2.6.32-600.el6'] == u.fixed_by('2.6.32-600.el6', introduced_in='2.6.32-')
        assert len(w) == 4

        rt_u = uname.Uname.from_uname_str("Linux qqhrycsq2 4.18.0-305.rt7.72.el8.x86_64 #1 SMP Wed Jun 13 18:24:36 EDT 2012 x86_64 x86_64 x86_64 GNU/Linux")
        # fixes without distribution part
        assert [] == rt_u.fixed_by('4.18.0-305.rt7.72')
        assert len(w) == 5

        # introduce without distribution part
        assert ['4.18.0-307.rt7.72.el8'] == rt_u.fixed_by('4.18.0-307.rt7.72.el8', introduced_in='4.18.0-305.rt7.72')
        assert len(w) == 6

    warnings.resetwarnings()


def test_unknown_release():
    u = uname.Uname.from_kernel("2.6.23-504.23.3.el6.revertBZ1169225")
    assert "504.23.3.0.el6" == u._lv_release


def test_fixed_by_rhel5():
    test_kernels = [
        (uname.Uname.from_uname_str("Linux oprddb1r5.example.com 2.6.18-348.el5 #1 SMP Wed Nov 28 21:22:00 EST 2012 x86_64 x86_64 x86_64 GNU/Linux"), []),
        (uname.Uname.from_uname_str("Linux srspidr1-3.example2.com 2.6.18-402.el5 #1 SMP Thu Jan 8 06:22:34 EST 2015 x86_64 x86_64 x86_64 GNU/Linux"), []),
        (uname.Uname.from_uname_str("Linux PVT-Dev1.pvtsolar.local 2.6.18-398.el5 #1 SMP Tue Aug 12 06:30:31 EDT 2014 x86_64 x86_64 x86_64 GNU/Linux"), []),
        (uname.Uname.from_kernel("2.6.18-194.el5"),
            ["2.6.18-238.40.1.el5", "2.6.18-308.13.1.el5", "2.6.18-348.el5"]),
        (uname.Uname.from_kernel("2.6.18-128.el5"),
            ["2.6.18-128.39.1.el5", "2.6.18-238.40.1.el5", "2.6.18-308.13.1.el5", "2.6.18-348.el5"]),

    ]
    for u, expected in test_kernels:
        fixed_by = u.fixed_by("2.6.18-128.39.1.el5", "2.6.18-238.40.1.el5", "2.6.18-308.13.1.el5", "2.6.18-348.el5")
        assert expected == fixed_by


def test_from_kernel_partial():
    u = uname.Uname.from_kernel("5.14.0-284.13.1")
    assert "284.13.1.0.0" == u._lv_release

    u = uname.Uname.from_kernel("5.14.0-284")
    assert "284.0.0.0.0" == u._lv_release

    early_rhel9 = uname.Uname.from_uname_str("Linux example 5.14.0-70.22.1.el9_0.x86_64 #1 SMP Wed May 24 08:55:08 UTC 2023 x86_64 x86_64 x86_64 GNU/Linux")
    later_rhel9_1 = uname.Uname.from_kernel("5.14.0-162.18.1")
    later_rhel9_2 = uname.Uname.from_kernel("5.14.0-284")

    # Test comparison of full uname with partial kernel version
    assert early_rhel9 < later_rhel9_1

    # Test comparison of partial kernel versions
    assert early_rhel9 < later_rhel9_2
    assert later_rhel9_1 < later_rhel9_2


def test_from_release():
    release = ("6", "4")
    from_release = uname.Uname.from_release(release)
    from_nvr = uname.Uname.from_kernel("2.6.32-358")
    assert str(from_release) == str(from_nvr)

    # Test the regular 7.4 version.
    release = ("7", "4")
    from_release = uname.Uname.from_release(release)
    from_nvr = uname.Uname.from_kernel("3.10.0-693")
    assert str(from_release) == str(from_nvr)

    # Test the kernel-alt 7.4 version.
    release = ("7", "4", "alt")
    from_release = uname.Uname.from_release(release)
    from_nvr = uname.Uname.from_kernel("4.11.0-44")
    assert str(from_release) == str(from_nvr)

    unknown_list = ["2.4.21-3", "2.6.9-4", "2.6.18-7", "2.6.32-70", "3.10.0-53"]
    known_list = [{'version': "2.4.21-4", 'rhel_release': ["3", "0"]},
                  {'version': "2.6.9-55", 'rhel_release': ["4", "5"]},
                  {'version': "2.6.18-308", 'rhel_release': ["5", "8"]},
                  {'version': "2.6.32-131.0.15", 'rhel_release': ["6", "1"]},
                  {'version': "3.10.0-123", 'rhel_release': ["7", "0"]}]
    for unknown_ver in unknown_list:
        unknown_uname = uname.Uname.from_uname_str("Linux hostname {version} #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux".format(version=unknown_ver))
        assert unknown_uname.rhel_release == ["-1", "-1"]
    for known_ver in known_list:
        known_uname = uname.Uname.from_uname_str("Linux hostname {version} #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux".format(version=known_ver['version']))
        assert known_uname.rhel_release == known_ver['rhel_release']


def test_uname_version_comparisons():
    # ### Test comparisons between Uname objects at the version level ###
    left__str = "Linux hostname 3.10.0-123.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    right_str = "Linux hostname 3.16.2-200.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    left = uname.Uname.from_uname_str(left__str)
    left_copy = uname.Uname.from_uname_str(left__str)
    right = uname.Uname.from_uname_str(right_str)
    right_copy = uname.Uname.from_uname_str(right_str)

    # Equality tests
    assert not left == right
    assert left == left_copy
    assert right == right_copy
    # Inequality tests
    assert left != right
    assert not left != left_copy
    assert not right != right_copy
    # Less than tests
    assert left < right
    assert not right < left
    assert not left < left_copy
    assert not right < right_copy
    # Less than or equal to tests
    assert left <= right
    assert not right <= left
    assert left <= left_copy
    assert right <= right_copy
    # Greater than tests
    assert not left > right
    assert right > left
    assert not left > left_copy
    assert not right > right_copy
    # Greater than or equal to tests
    assert not left >= right
    assert right >= left
    assert left >= left_copy
    assert right >= right_copy


def test_uname_release_comparisons():
    # ### Test comparisons between Uname objects at the release level ###
    left__str = "Linux hostname 3.16.2-200.10.1.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    right_str = "Linux hostname 3.16.2-200.9.1.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    left = uname.Uname.from_uname_str(left__str)
    left_copy = uname.Uname.from_uname_str(left__str)
    right = uname.Uname.from_uname_str(right_str)
    right_copy = uname.Uname.from_uname_str(right_str)

    # Equality tests
    assert not left == right
    assert left == left_copy
    assert right == right_copy
    # Inequality tests
    assert left != right
    assert not left != left_copy
    assert not right != right_copy
    # Less than tests
    assert not left < right
    assert right < left
    assert not left < left_copy
    assert not right < right_copy
    # Less than or equal to tests
    assert not left <= right
    assert right <= left
    assert left <= left_copy
    assert right <= right_copy
    # Greater than tests
    assert left > right
    assert not right > left
    assert not left > left_copy
    assert not right > right_copy
    # Greater than or equal to tests
    assert left >= right
    assert not right >= left
    assert left >= left_copy
    assert right >= right_copy


def test_uname_vrastring_comparisons():
    # ### Test comparisons between Uname objects and ver-rel-arch strings ###
    left__str = "Linux hostname 3.16.2-200.10.1.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    left = uname.Uname.from_uname_str(left__str)
    left_ver_rel = "3.16.2-200.10.1.el7.x86_64"
    right_ver_rel = "3.16.2-200.9.1.el7.x86_64"
    # Equality tests
    assert not left == right_ver_rel
    assert left == left_ver_rel
    # Inequality tests
    assert left != right_ver_rel
    assert not left != left_ver_rel
    # Less than tests
    assert not left < right_ver_rel
    assert right_ver_rel < left
    assert not left < left_ver_rel
    # Less than or equal to tests
    assert not left <= right_ver_rel
    assert right_ver_rel <= left
    assert left <= left_ver_rel
    # Greater than tests
    assert left > right_ver_rel
    assert not right_ver_rel > left
    assert not left > left_ver_rel
    # Greater than or equal to tests
    assert left >= right_ver_rel
    assert not right_ver_rel >= left
    assert left >= left_ver_rel


def test_uname_release_length_comparisons():
    # ### Test comparisons between Uname objects where releases loosely match ###
    minor_left__str = "Linux hostname 3.16.2-200.0.0.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    minor_right_str = "Linux hostname 3.16.2-200.el7.x86_64 #1 SMP Mon Sep 8 11:54:45 UTC 2014 x86_64 x86_64 x86_64 GNU/Linux"
    left = uname.Uname.from_uname_str(minor_left__str)
    right = uname.Uname.from_uname_str(minor_right_str)
    # Equality tests
    assert left == right
    # Inequality tests
    assert not left != right
    # Less than tests
    assert not left < right
    assert not right < left
    # Less than or equal to tests
    assert left <= right
    assert right <= left
    # Greater than tests
    assert not left > right
    assert not right > left
    # Greater than or equal to tests
    assert left >= right
    assert right >= left


def test_docker_uname():
    u = uname.Uname.from_uname_str("Linux 06a04d0354dc 4.0.3-boot2docker #1 SMP Wed May 13 20:54:49 UTC 2015 x86_64 x86_64 x86_64 GNU/Linux")
    assert "boot2docker" == u.release


uname_line = 'Linux server1.example.com 2.6.32-504.el6.x86_64 #1 SMP Tue Sep 16 01:56:35 EDT 2014 x86_64 x86_64 x86_64 GNU/Linux'


# Because tests are done at the module level, we have to put all the shared
# parser information in the one environment.  Fortunately this is normal.
def test_uname_doc_examples():
    env = {
        'Uname': uname.Uname,
        'uname': uname.Uname(context_wrap(uname_line)),
    }
    failed, total = doctest.testmod(uname, globs=env)
    assert failed == 0



from insights.parsers.ls_disk import LsDisk
from insights.tests import context_wrap

LS_DISK = """
/dev/disk/by-path:
total 0
drwxr-xr-x. 2 0 0 160 Sep 19 10:15 .
drwxr-xr-x. 5 0 0 100 Sep 19 10:15 ..
lrwxrwxrwx. 1 0 0   9 Sep 19 10:15 pci-0000:00:01.1-scsi-1:0:0:0 -> ../../sr0
lrwxrwxrwx. 1 0 0   9 Sep 19 10:15 pci-0000:00:0d.0-scsi-0:0:0:0 -> ../../sda
lrwxrwxrwx. 1 0 0  10 Sep 19 10:15 pci-0000:00:0d.0-scsi-0:0:0:0-part1 -> ../../sda1
lrwxrwxrwx. 1 0 0  10 Sep 19 10:15 pci-0000:00:0d.0-scsi-0:0:0:0-part2 -> ../../sda2
lrwxrwxrwx. 1 0 0   9 Sep 19 10:15 pci-0000:00:0d.0-scsi-1:0:0:0 -> ../../sdb
lrwxrwxrwx. 1 0 0  10 Sep 19 10:15 pci-0000:00:0d.0-scsi-1:0:0:0-part1 -> ../../sdb

/dev/disk/by-uuid:
total 0
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 3ab50b34-d0b9-4518-9f21-05307d895f81 -> ../../dm-1
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 51c5cf12-a577-441e-89da-bc93a73a1ba3 -> ../../sda1
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 7b0068d4-1399-4ce7-a54a-3e2fc1232299 -> ../../dm-0

/dev/disk/by-id:
total 0
lrwxrwxrwx. 1 root root  9 Sep 19 10:15 ata-VBOX_CD-ROM_VB2-01700376 -> ../../sr0
lrwxrwxrwx. 1 root root  9 Sep 19 10:15 ata-VBOX_HARDDISK_VB4c56cb04-26932e6a -> ../../sdb
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 ata-VBOX_HARDDISK_VB4c56cb04-26932e6a-part1 -> ../../sdb1
lrwxrwxrwx. 1 root root  9 Sep 19 10:15 ata-VBOX_HARDDISK_VBdb3eca59-6c439fb7 -> ../../sda
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 ata-VBOX_HARDDISK_VBdb3eca59-6c439fb7-part1 -> ../../sda1
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 ata-VBOX_HARDDISK_VBdb3eca59-6c439fb7-part2 -> ../../sda2
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 dm-name-vg_yizhangrhel6-lv_root -> ../../dm-0
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 dm-name-vg_yizhangrhel6-lv_swap -> ../../dm-1
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 dm-uuid-LVM-vT3Sj2vOtGXyChqSzGvmps0qezNWEO7MI7atvuhelVCBetzJ3QPpOlIDyXSIt6cw -> ../../dm-1
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 dm-uuid-LVM-vT3Sj2vOtGXyChqSzGvmps0qezNWEO7MlJOewBrdU3NOoolrIVn99sdXojq9qIxd -> ../../dm-0
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 lvm-pv-uuid-1lI8Ef-9H35-HVlP-1UdC-XlgD-eedv-SRjC0i -> ../../sda2
lrwxrwxrwx. 1 root root  9 Sep 19 10:15 scsi-SATA_VBOX_HARDDISK_VB4c56cb04-26932e6a -> ../../sdb
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 scsi-SATA_VBOX_HARDDISK_VB4c56cb04-26932e6a-part1 -> ../../sdb1
lrwxrwxrwx. 1 root root  9 Sep 19 10:15 scsi-SATA_VBOX_HARDDISK_VBdb3eca59-6c439fb7 -> ../../sda
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 scsi-SATA_VBOX_HARDDISK_VBdb3eca59-6c439fb7-part1 -> ../../sda1
lrwxrwxrwx. 1 root root 10 Sep 19 10:15 scsi-SATA_VBOX_HARDDISK_VBdb3eca59-6c439fb7-part2 -> ../../sda2

/dev/disk/by-label:
total 0
lrwxrwxrwx. 1 root root 9 Sep 18 11:20 RHEL-7.2\x20Server.x86_64 -> ../../sr0
"""

BY_DISK = "/dev/disk/by-path"


def test_ls_disk():
    ls_disk = LsDisk(context_wrap(LS_DISK))
    assert ls_disk.listing_of("/dev/disk/by-uuid") == {
        '7b0068d4-1399-4ce7-a54a-3e2fc1232299': {'group': 'root', 'name': '7b0068d4-1399-4ce7-a54a-3e2fc1232299',
                                                 'links': 1, 'perms': 'rwxrwxrwx.', 'dir': '/dev/disk/by-uuid',
                                                 'raw_entry': 'lrwxrwxrwx. 1 root root 10 Sep 19 10:15 7b0068d4-1399-4ce7-a54a-3e2fc1232299 -> ../../dm-0',
                                                 'owner': 'root', 'link': '../../dm-0', 'date': 'Sep 19 10:15',
                                                 'type': 'l',
                                                 'size': 10},
        '3ab50b34-d0b9-4518-9f21-05307d895f81': {'group': 'root', 'name': '3ab50b34-d0b9-4518-9f21-05307d895f81',
                                                 'links': 1, 'perms': 'rwxrwxrwx.', 'dir': '/dev/disk/by-uuid',
                                                 'raw_entry': 'lrwxrwxrwx. 1 root root 10 Sep 19 10:15 3ab50b34-d0b9-4518-9f21-05307d895f81 -> ../../dm-1',
                                                 'owner': 'root', 'link': '../../dm-1', 'date': 'Sep 19 10:15',
                                                 'type': 'l',
                                                 'size': 10},
        '51c5cf12-a577-441e-89da-bc93a73a1ba3': {'group': 'root', 'name': '51c5cf12-a577-441e-89da-bc93a73a1ba3',
                                                 'links': 1, 'perms': 'rwxrwxrwx.', 'dir': '/dev/disk/by-uuid',
                                                 'raw_entry': 'lrwxrwxrwx. 1 root root 10 Sep 19 10:15 51c5cf12-a577-441e-89da-bc93a73a1ba3 -> ../../sda1',
                                                 'owner': 'root', 'link': '../../sda1', 'date': 'Sep 19 10:15',
                                                 'type': 'l',
                                                 'size': 10}}
    assert ls_disk.listings.get("/dev/disk/by-uuid")['files'] == ['3ab50b34-d0b9-4518-9f21-05307d895f81',
                                                                  '51c5cf12-a577-441e-89da-bc93a73a1ba3',
                                                                  '7b0068d4-1399-4ce7-a54a-3e2fc1232299']

    assert ls_disk.listings.get("/dev/disk/by-uuid")['entries']['3ab50b34-d0b9-4518-9f21-05307d895f81'][
               'link'] == "../../dm-1"



from insights.parsers import ls_krb5_sssd
from insights.parsers.ls_krb5_sssd import LsKrb5SSSD
from insights.tests import context_wrap
import doctest

LS_KRB5_SSSD = """
/var/lib/sss/pubconf/krb5.include.d:
total 24
drwxr-xr-x@ 6 501  20  192 Jul  1 23:46 .
drwxr-xr-x@ 3 501  20   96 Jul  1 23:48 ..
-rw-r--r--@ 1 501  20  674 Jul  1 23:46 domain_realm_rhidm_gwl_bz
-rw-r--r--@ 1 501  20   35 Jul  1 23:46 krb5_libdefaults
-rw-r--r--@ 1 501  20   98 Jul  1 23:46 localauth_plugin
-rw-------@ 1 501  20    0 Oct  1  2021 localauth_pluginolsIe3
"""


def test_ls_krb5_sssd():
    ls_krb5_sssd = LsKrb5SSSD(context_wrap(LS_KRB5_SSSD))
    assert '/var/lib/sss/pubconf/krb5.include.d' in ls_krb5_sssd
    assert ls_krb5_sssd.files_of('/var/lib/sss/pubconf/krb5.include.d') == ['domain_realm_rhidm_gwl_bz', 'krb5_libdefaults', 'localauth_plugin', 'localauth_pluginolsIe3']


def test_ls_krb5_sssd_doc():
    failed_count, tests = doctest.testmod(
        ls_krb5_sssd,
        globs={'ls_krb5_sssd': ls_krb5_sssd.LsKrb5SSSD(context_wrap(LS_KRB5_SSSD))}
    )
    assert failed_count == 0



"""
Haproxy configuration files
===========================

Parsers provided by this module are:

HaproxyCfg - file ``/var/lib/config-data/puppet-generated/haproxy/etc/haproxy/haproxy.cfg`` or ``/etc/haproxy/haproxy.cfg``
---------------------------------------------------------------------------------------------------------------------------
HaproxyCfgScl - file ``/etc/opt/rh/rh-haproxy18/haproxy/haproxy.cfg``
---------------------------------------------------------------------
"""

from insights.core import Parser
from insights.core.plugins import parser
from insights.specs import Specs


class HaproxyFile(Parser):
    """
    Base class for ``HaproxyCfg`` and ``HaproxyCfgScl`` classes.

    Attributes:
        data (dict): Dictionary of all parsed sections.
        lines (list): List of all non-commented lines.

    Content of the `haproxy.cfg` file looks like::

        global
            daemon
            group       haproxy
            log         /dev/log local0
            user        haproxy
            maxconn     20480
            pidfile     /var/run/haproxy.pid

        defaults
            retries     3
            maxconn     4096
            log         global
            timeout     http-request 10s
            timeout     queue 1m
            timeout     connect 10s

    Examples:
        >>> type(haproxy)
        <class 'insights.parsers.haproxy_cfg.HaproxyFile'>
        >>> haproxy.data['global']
        {'daemon': '', 'group': 'haproxy', 'log': '/dev/log local0', 'user': 'haproxy', 'maxconn': '20480', 'pidfile': '/var/run/haproxy.pid'}
        >>> haproxy.data['global']['group']
        'haproxy'
        >>> 'global' in haproxy.data
        True
        >>> 'user' in haproxy.data.get('global')
        True
        >>> haproxy.data['defaults']
        {'retries': '3', 'maxconn': '4096', 'log': 'global', 'timeout': ['http-request 10s', 'queue 1m', 'connect 10s']}
    """

    SECTION_NAMES = ("global", "defaults", "frontend", "backend", "listen")

    def __init__(self, context):
        self.data = {}
        self.lines = []
        super(HaproxyFile, self).__init__(context)

    def parse_content(self, content):
        section_dict = {}

        for line in content:
            line = line.strip()
            if line.startswith("#") or line == "":
                continue

            self.lines.append(line)
            split_items = line.split(None, 1)

            # Create a new section, e.g. global: {}
            if split_items[0] in self.SECTION_NAMES:
                section_dict = {}
                section = split_items[0] if len(split_items) == 1 else split_items[0] + " " + split_items[1]
                self.data.update({section: section_dict})
            # Handle attributes inside section
            else:
                if len(split_items) == 1:
                    section_dict[line] = ""
                else:
                    key = split_items[0]
                    value = split_items[1]
                    if key in section_dict:
                        # Convert value into list in case of duplicate key items
                        if not isinstance(section_dict[key], list):
                            section_dict[key] = [section_dict[key]]
                        section_dict[key].append(value)
                    else:
                        section_dict[key] = value


@parser(Specs.haproxy_cfg)
class HaproxyCfg(HaproxyFile):
    """
    Class to parse file ``/var/lib/config-data/puppet-generated/haproxy/etc/haproxy/haproxy.cfg`` or ``haproxy.cfg``.
    """
    pass


@parser(Specs.haproxy_cfg_scl)
class HaproxyCfgScl(HaproxyFile):
    """
    Class to parse file ``/etc/opt/rh/rh-haproxy18/haproxy/haproxy.cfg``.
    """
    pass



from insights.tests import context_wrap
from insights.parsers.dmesg_log import DmesgLog

DMESG_EXAMPLE = """
[    0.000000] Initializing cgroup subsys cpu
[    0.000000] Linux version 3.10.0-862.el7.x86_64 (mockbuild@x86-034.build.eng.bos.redhat.com) \
(gcc version 4.8.5 20150623 (Red Hat 4.8.5-28) (GCC) ) #1 SMP Wed Mar 21 18:14:51 EDT 2018
[    2.090905] SELinux:  Completing initialization.
[    2.090907] SELinux:  Setting up existing superblocks.
[    2.099684] systemd[1]: Successfully loaded SELinux policy in 82.788ms.
[    2.117410] ip_tables: (C) 2000-2006 Netfilter Core Team
[    2.117429] systemd[1]: Inserted module 'ip_tables'
[    2.376551] systemd-journald[441]: Received request to flush runtime journal from PID 1
[    2.716874] cryptd: max_cpu_qlen set to 100
[    2.804152] AES CTR mode by8 optimization enabled
[    2.874051] XFS (vda1): Mounting V5 Filesystem
[    2.919331] alg: No test for __gcm-aes-aesni (__driver-gcm-aes-aesni)
[    2.939496] [TTM] Zone  kernel: Available graphics memory: 507714 kiB
[    2.939498] [TTM] Initializing pool allocator
[    2.939502] [TTM] Initializing DMA pool allocator
""".strip()


def test_dmesg():
    dmesg_log = DmesgLog(context_wrap(DMESG_EXAMPLE))
    assert dmesg_log.get("SELinux")[1]["raw_message"] == "[    2.090907] SELinux:  Setting up existing superblocks."
    assert len(dmesg_log.get("TTM")) == 3
    assert "XFS" in dmesg_log
    assert "hpsa" not in dmesg_log
    assert dmesg_log.has_startswith("ip_tables")
    assert not dmesg_log.has_startswith("policy")
    assert dmesg_log.logs_startwith("cryptd") == ["cryptd: max_cpu_qlen set to 100"]
    assert dmesg_log.logs_startwith("perf") == []
    assert len(list(dmesg_log.get_after(2.919331))) == 4
    assert len(list(dmesg_log.get_after(2.919331, "TTM"))) == 3



"""
AuditCtl - command ``auditctl xxx``
===================================

This module contains the following parsers:

AuditRules - command ``auditctl -l``
------------------------------------
AuditStatus - command ``auditctl -s``
-------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.auditctl_rules)
class AuditRules(CommandParser, list):
    """
    Class for parsing the `auditctl -l` command.
    All lines are stored in a list.

    Typical output of the command is::

        -w /etc/selinux -p wa -k MAC-policy
        -a always,exit -F arch=b32 -S chmod -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b64 -S chmod -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b32 -S chmod -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b64 -S chmod -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b32 -S chown -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b64 -S chown -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b32 -S chown -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access
        -a always,exit -F arch=b64 -S chown -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access

    Examples:
        >>> type(audit_rules)
        <class 'insights.parsers.auditctl.AuditRules'>
        >>> len(audit_rules)
        9
        >>> '-w /etc/selinux -p wa -k MAC-policy' in audit_rules
        True

    Raises:
        SkipComponent: When there are not rules.
    """

    def parse_content(self, content):
        if len(content) == 1 and content[0].lower().strip() == 'no rules':
            raise SkipComponent
        for line in content:
            if line.strip():
                self.append(line.strip())
        if not self:
            raise SkipComponent('No rules found')


@parser(Specs.auditctl_status)
class AuditStatus(CommandParser, dict):
    """
    Module for parsing the output of the ``auditctl -s`` command.

    Typical output on RHEL6 looks like::

        AUDIT_STATUS: enabled=1 flag=1 pid=1483 rate_limit=0 backlog_limit=8192 lost=3 backlog=0

    , while on RHEL7 and later, the output changes to::

        enabled 1
        failure 1
        pid 947
        rate_limit 0
        backlog_limit 320
        lost 0
        backlog 0
        loginuid_immutable 0 unlocked

    Example:
        >>> type(auds)
        <class 'insights.parsers.auditctl.AuditStatus'>
        >>> "enabled" in auds
        True
        >>> auds['enabled']
        1
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("Input content is empty.")
        if len(content) > 1:
            for line in content:
                k, v = line.split(None, 1)
                # Mind the 'loginuid_immutable' on RHEL7
                if k.strip() == "loginuid_immutable":
                    self[k.strip()] = v.strip()
                else:
                    try:
                        self[k.strip()] = int(v.strip())
                    except ValueError:
                        raise ParseException('Unexpected type in line %s' % line)
        if len(content) == 1:
            line = list(content)[0].strip()
            if line.startswith("AUDIT_STATUS:"):
                for item in line.split(None)[1:]:
                    try:
                        k, v = item.split('=')
                        self[k.strip()] = int(v.strip())
                    except ValueError:
                        raise ParseException('Unexpected type in line %s ' % line)
        if not self:
            raise SkipComponent('There is no content in the status output.')



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import bootc
from insights.parsers.bootc import BootcStatus
from insights.tests import context_wrap

BOOTC_STATUS = '''
{
   "apiVersion":"org.containers.bootc/v1alpha1",
   "kind":"BootcHost",
   "metadata":{
      "name":"host"
   },
   "spec":{
      "image":{
         "image":"192.168.124.1:5000/bootc-insights:latest",
         "transport":"registry"
      }
   },
   "status":{
      "staged":null,
      "booted":{
         "image":{
            "image":{
               "image":"192.168.124.1:5000/bootc-insights:latest",
               "transport":"registry"
            },
            "version":"stream9.20231213.0",
            "timestamp":null,
            "imageDigest":"sha256:806d77394f96e47cf99b1233561ce970c94521244a2d8f2affa12c3261961223"
         },
         "incompatible":false,
         "pinned":false,
         "ostree":{
            "checksum":"6aa32a312c832e32a2dbfe006f05e5972d9f2b86df54e747128c24e6c1fb129a",
            "deploySerial":0
         }
      },
      "rollback":{
         "image":{
            "image":{
               "image":"quay.io/centos-boot/fedora-boot-cloud:eln",
               "transport":"registry"
            },
            "version":"39.20231109.3",
            "timestamp":null,
            "imageDigest":"sha256:92e476435ced1c148350c660b09c744717defbd300a15d33deda5b50ad6b21a0"
         },
         "incompatible":false,
         "pinned":false,
         "ostree":{
            "checksum":"56612a5982b7f12530988c970d750f89b0489f1f9bebf9c2a54244757e184dd8",
            "deploySerial":0
         }
      },
      "type":"bootcHost"
   }
}
'''.strip()


def test_bootc_doc_examples():
    env = {
        "bootc_status": BootcStatus(context_wrap(BOOTC_STATUS)),
    }
    failed, total = doctest.testmod(bootc, globs=env)
    assert failed == 0


def test_bootc_status_skip():
    with pytest.raises(SkipComponent) as ex:
        BootcStatus(context_wrap(""))
    assert "Empty output." in str(ex)


def test_bootc_status():
    bs = BootcStatus(context_wrap(BOOTC_STATUS))
    assert bs['apiVersion'] == 'org.containers.bootc/v1alpha1'
    assert bs['status']['booted']['image']['version'] == 'stream9.20231213.0'



"""
Custom datasource to get the linux users count who map to a selinux user.
"""
import json

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.parsers import parse_fixed_table
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by users_count_map_selinux_user datasource """

    selinux_user_mapping = simple_command("/sbin/semanage login -l")


@datasource(LocalSpecs.selinux_user_mapping, HostContext)
def users_count_map_selinux_user(broker):
    """
    Get linux users count who map to the selinux user. It returns a dict with
    the selinux user type as the key, and the linux users count as the value.

    Raises:
        SkipComponent: Raised when there are no users map the filtered selinux user.
    """
    user_mapping_lines = broker[LocalSpecs.selinux_user_mapping].content
    data = parse_fixed_table(
        user_mapping_lines,
        heading_ignore=['Login Name'],
        header_substitute=[
            ('Login Name', 'login_name'),
            ('SELinux User', 'selinux_user'),
            ('MLS/MCS Range', 'mls_mcs_range'),
            ('Service', 'service'),
        ]
    )
    filters = sorted((get_filters(Specs.selinux_users)))
    users_info = {}
    for filter in filters:
        users_info[filter] = len([obj for obj in data if obj['selinux_user'] == filter])
    if users_info:
        return DatasourceProvider(json.dumps(users_info), relative_path='insights_commands/linux_users_count_map_selinux_user')
    raise SkipComponent



# -*- coding: UTF-8 -*-
import six
from insights.core.ls_parser import parse


SINGLE_DIRECTORY = """
total 32
drwxr-xr-x.  5 root root  4096 Jun 28  2017 .
drwxr-xr-x. 15 root root  4096 Aug 10 09:42 ..
lrwxrwxrwx.  1 root root    49 Jun 28  2017 cert.pem -> /etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem
drwxr-xr-x.  2 root root  4096 Jun 28  2017 certs
drwxr-xr-x.  2 root root  4096 Mar 29  2017 misc
-rw-r--r--.  1 root root 10923 Feb  7  2017 openssl.cnf
drwxr-xr-x.  2 root root  4096 Feb  7  2017 private
"""

SINGLE_DIRECTORY_WITH_ENTRY = """
/etc:
total 32
drwxr-xr-x.  5 root root  4096 Jun 28  2017 .
drwxr-xr-x. 15 root root  4096 Aug 10 09:42 ..
lrwxrwxrwx.  1 root root    49 Jun 28  2017 cert.pem -> /etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem
drwxr-xr-x.  2 root root  4096 Jun 28  2017 certs
drwxr-xr-x.  2 root root  4096 Mar 29  2017 misc
-rw-r--r--.  1 root root 10923 Feb  7  2017 openssl.cnf
drwxr-xr-x.  2 root root  4096 Feb  7  2017 private
"""

MULTIPLE_DIRECTORIES = """
ls: cannot access '_non_existing_': No such file or directory
/etc/sysconfig:
total 96
drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
-rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
-rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub

/etc/rc.d/rc3.d:
total 4
drwxr-xr-x.  2 0 0   58 Jul  6 23:32 .
drwxr-xr-x. 10 0 0 4096 Sep 16  2015 ..
lrwxrwxrwx.  1 0 0   20 Jul  6 23:32 K50netconsole -> ../init.d/netconsole
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 S10network -> ../init.d/network
lrwxrwxrwx.  1 0 0   15 Jul  6 23:32 S97rhnsd -> ../init.d/rhnsd
"""

MULTIPLE_DIRECTORIES_WITH_BREAK = """
ls: cannot access '_non_existing_': No such file or directory
/etc:
total 1652
drwxr-xr-x. 102   0   0  12288 Nov  6 09:12 .
dr-xr-xr-x.  21   0   0   4096 Oct 23 10:12 ..
-rw-------.   1   0   0      0 Oct  2 13:46 .pwd.lock
-rw-r--r--    1   0   0    163 Oct  2 13:45 .updated
-rw-r--r--.   1   0   0   1059 Oct  2 13:56 chrony.conf
-rw-r--r--.   1   0   0   1100 Dec  5  2017 chrony.conf.20180210135613

-rw-r-----.   1   0 993    481 Sep 14  2017 chrony.keys
drwxr-xr-x.   2   0   0   4096 Nov  1 03:34 cifs-utils

/etc/sysconfig:
total 96
drwxr-xr-x.  7 0 0 4096 Jul  6 23:41 .
drwxr-xr-x. 77 0 0 8192 Jul 13 03:55 ..
drwxr-xr-x.  2 0 0   41 Jul  6 23:32 cbq
drwxr-xr-x.  2 0 0    6 Sep 16  2015 console
-rw-------.  1 0 0 1390 Mar  4  2014 ebtables-config
-rw-r--r--.  1 0 0   72 Sep 15  2015 firewalld
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 grub -> /etc/default/grub

/etc/rc.d/rc3.d:
total 4
drwxr-xr-x.  2 0 0   58 Jul  6 23:32 .
drwxr-xr-x. 10 0 0 4096 Sep 16  2015 ..
lrwxrwxrwx.  1 0 0   20 Jul  6 23:32 K50netconsole -> ../init.d/netconsole
lrwxrwxrwx.  1 0 0   17 Jul  6 23:32 S10network -> ../init.d/network
lrwxrwxrwx.  1 0 0   15 Jul  6 23:32 S97rhnsd -> ../init.d/rhnsd
"""


COMPLICATED_FILES = """
ls: cannot access '_non_existing_': No such file or directory
/tmp:
total 16
dr-xr-xr-x.  2 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 10 0 0     4096 Mar  4 16:19 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
lrwxrwxrwx.  1 0 0       11 Aug  4  2014 menu.lst -> ./grub.conf
brw-rw----.  1 0 6 253,  10 Aug  4 16:56 dm-10
crw-------.  1 0 0 10,  236 Jul 25 10:00 control
srw-------.  1 26214 17738 0 Oct 19 08:48 geany_socket.c46453c2
-rw-rw-r--.  1 24306 24306 13895 Oct 21 15:42 File name with spaces in it!
-rw-rw-r--.  1 24306 24306 13895 Oct 21 15:42 Unicode ÅÍÎÏÓÔÒÚÆ☃ madness.txt
drwxr-xr-x+  2 0 0       41 Jul  6 23:32 additional_ACLs
brw-rw----.  1 0 6  1048576 Aug  4 16:56 block dev with no comma also valid
-rwxr-xr-x.  2 0 0     1024 Jul  6 23:32 file_name_ending_with_colon:
lrwxrwxrwx.  1 0 0       11 Aug  4  2014 link with spaces -> ../file with spaces
"""

COMPLICATED_FILES_BAD_LINE = """
/tmp:
total 16
dr-xr-xr-x.  2 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 10 0 0     4096 Mar  4 16:19 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
ls: cannot open directory '/etc/audisp': Permission denied
lrwxrwxrwx.  1 0 0       11 Aug  4  2014 menu.lst -> ./grub.conf
brw-rw----.  1 0 6 253,  10 Aug  4 16:56 dm-10
crw-------.  1 0 0 10,  236 Jul 25 10:00 control
srw-------.  1 26214 17738 0 Oct 19 08:48 geany_socket.c46453c2
-rw-rw-r--.  1 24306 24306 13895 Oct 21 15:42 File name with spaces in it!
-rw-rw-r--.  1 24306 24306 13895 Oct 21 15:42 Unicode ÅÍÎÏÓÔÒÚÆ☃ madness.txt
drwxr-xr-x+  2 0 0       41 Jul  6 23:32 additional_ACLs
brw-rw----.  1 0 6  1048576 Aug  4 16:56 block dev with no comma also valid
-rwxr-xr-x.  2 0 0     1024 Jul  6 23:32 file_name_ending_with_colon:
lrwxrwxrwx.  1 0 0       11 Aug  4  2014 link with spaces -> ../file with spaces
"""

SELINUX_DIRECTORY = """
/boot:
total 3
-rw-r--r--. root root system_u:object_r:boot_t:s0      config-3.10.0-267
drwxr-xr-x. root root system_u:object_r:boot_t:s0      grub2
-rw-r--r--. root root system_u:object_r:boot_t:s0      initramfs-0-rescue
"""

RHEL8_SELINUX_DIRECTORY = """
/var/lib/nova/instances:
total 0
drwxr-xr-x. 3 root root unconfined_u:object_r:var_lib_t:s0 50 Apr  8 16:41 .
drwxr-xr-x. 3 root root unconfined_u:object_r:var_lib_t:s0 23 Apr  8 16:29 ..
drwxr-xr-x. 2 root root unconfined_u:object_r:var_lib_t:s0 54 Apr  8 16:41 abcd-efgh-ijkl-mnop
"""

FILES_CREATED_WITH_SELINUX_DISABLED = """
/dev/mapper:
total 2
lrwxrwxrwx 1 0 0 7 Apr 27 05:34 lv_cpwtk001_data01 -> ../dm-7
lrwxrwxrwx 1 0 0 7 Apr 27 05:34 lv_cpwtk001_redo01 -> ../dm-8
"""

BAD_DIRECTORY_ENTRIES = """
dr-xr-xr-x.  2 0 0     4096 Mar  4 16:19 dir entry with no dir header
total 3

/badness:
    -rwxr-xr-x. 0 0    1 Sep 12 2010 indented entry
xr-xr--r--. 0 0        1 Sep 12  2010 bad file type
-rxr-xr-x.  0 0        1 Sep 12  2010 missing user w permission
-rwxr-xr-x  0 0        1 Sep 12  2010 missing ACL dot
-rw-r--r--. user with spaces group 2 Oct 3 2011 user with spaces
-rw-r--r--. user group with spaces 2 Oct 3 2011 group with spaces
dr-xr-xr-x. -42 -63 1271 Jan  6  2008 Negative user and group numbers
dr-xr-xr-x. 1 7 123, 124, 125 Jan 6 2008 Three comma blocks in size
brw-rw----. 1 0 6 123456 Aug 4 16:56 two size blocks
prw-rw----. 1000 1000  0  6 2007 Month missing
prw-rw----. 1000 1000  0 No 6 2007 Month too short
prw-rw----. 1000 1000  0 November 6 2007 Month too long
prw-rw----. 1000 1000  0 Nov  2007 Day too long
prw-rw----. 1000 1000  0 Nov 126 2007 Day too long
prw-rw----. 1000 1000  0 Nov 126  Year missing
prw-rw----. 1000 1000  0 Nov 126 20107 Year too long
prw-rw----. 1000 1000  0 Nov 12 :56 Missing hour
prw-rw----. 1000 1000  0 Nov 12 723:56 Hour too long
prw-rw----. 1000 1000  0 Nov 12 23: Missing minute
prw-rw----. 1000 1000  0 Nov 12 23:3 Minute too short
prw-rw----. 1000 1000  0 Nov 12 23:357 Minute too long
-rw------ 1 root root 762 Sep 23 002 permission too short
bash: ls: command not found
-rw------ 1 root root 762 Se
-rw------- 1 ro:t root 762 Sep 23 002 colon in uid
-rw------- 1 root r:ot 762 Sep 23 002 colon in gid
-rwasdfas- 1 root root 762 Sep 23 002 bad permissions block
-rwx/----- 1 root root 762 Sep 23 002 slash in permissions block
-rwx------ 1 root root 762 Sep 23 002 /slashes/in/filename
/rwasdfas- 1 root root 762 Sep 23 002 slash in file type and no colon on end
/usr/bin/ls: cannot access /boot/grub2/grub.cfg: No such file or directory
cannot access /boot/grub2/grub.cfg: No such file or directory
No such file or directory
adsf
"""


def test_parse_selinux():
    results = parse(SELINUX_DIRECTORY.splitlines(), "/boot")
    stanza = results["/boot"]
    assert stanza["name"] == "/boot"
    assert stanza["total"] == 3
    assert len(stanza["entries"]) == 3
    res = stanza["entries"]["config-3.10.0-267"]
    assert res["type"] == "-"
    assert res["owner"] == "root"
    assert res["group"] == "root"
    assert res["se_user"] == "system_u"
    assert res["se_role"] == "object_r"
    assert res["se_type"] == "boot_t"
    assert res["se_mls"] == "s0"
    assert res["name"] == "config-3.10.0-267"


def test_parse_single_directory():
    results = parse(SINGLE_DIRECTORY.splitlines(), "/etc")
    stanza = results["/etc"]
    assert stanza["name"] == "/etc"
    assert stanza["total"] == 32
    assert len(stanza["entries"]) == 7
    res = stanza["entries"]["cert.pem"]
    assert res["type"] == "l"
    assert res["owner"] == "root"
    assert res["group"] == "root"
    assert res["date"] == "Jun 28  2017"
    assert res["name"] == "cert.pem"
    assert res["link"] == "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem"
    assert res["links"] == 1
    assert res["dir"] == "/etc"

    results = parse(SINGLE_DIRECTORY_WITH_ENTRY.splitlines())
    stanza = results["/etc"]
    assert stanza["name"] == "/etc"
    assert stanza["total"] == 32
    assert len(stanza["entries"]) == 7
    res = stanza["entries"]["cert.pem"]
    assert res["type"] == "l"
    assert res["owner"] == "root"
    assert res["group"] == "root"
    assert res["date"] == "Jun 28  2017"
    assert res["name"] == "cert.pem"
    assert res["link"] == "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem"
    assert res["links"] == 1
    assert res["dir"] == "/etc"


def test_parse_multiple_directories():
    results = parse(MULTIPLE_DIRECTORIES.splitlines(), None)
    assert len(results) == 2, len(results)
    assert results["/etc/sysconfig"]["name"] == "/etc/sysconfig"
    assert results["/etc/sysconfig"]["total"] == 96
    assert results["/etc/rc.d/rc3.d"]["name"] == "/etc/rc.d/rc3.d"
    assert results["/etc/rc.d/rc3.d"]["total"] == 4

    res = results["/etc/sysconfig"]["entries"]["ebtables-config"]
    assert res["type"] == "-"
    assert res["links"] == 1
    assert res["owner"] == "0"
    assert res["group"] == "0"
    assert res["size"] == 1390
    assert res["date"] == "Mar  4  2014"
    assert res["name"] == "ebtables-config"
    assert res["dir"] == "/etc/sysconfig"


def test_parse_multiple_directories_with_break():
    results = parse(MULTIPLE_DIRECTORIES_WITH_BREAK.splitlines(), None)
    assert len(results) == 3, len(results)
    assert len(results.values()) == 3
    assert len(results.items()) == 3
    assert len(list(six.iteritems(results))) == 3
    assert results["/etc"]["name"] == "/etc"
    assert results["/etc"]["total"] == 1652
    assert results["/etc/rc.d/rc3.d"]["name"] == "/etc/rc.d/rc3.d"
    assert results["/etc/rc.d/rc3.d"]["total"] == 4
    assert 'chrony.keys' in results['/etc']['files']
    assert 'cifs-utils' in results['/etc']['dirs']

    res = results["/etc"]["entries"]["chrony.conf.20180210135613"]
    assert res["type"] == "-"
    assert res["links"] == 1
    assert res["owner"] == "0"
    assert res["group"] == "0"
    assert res["size"] == 1100
    assert res["date"] == "Dec  5  2017"
    assert res["name"] == "chrony.conf.20180210135613"
    assert res["dir"] == "/etc"


def test_complicated_files():
    results = parse(COMPLICATED_FILES.splitlines(), "/tmp")
    assert len(results) == 1
    assert results["/tmp"]["total"] == 16, results["/tmp"]["total"]
    assert results["/tmp"]["name"] == "/tmp", results["/tmp"]["name"]
    res = results["/tmp"]["entries"]["dm-10"]
    assert res["type"] == "b"
    assert res["links"] == 1
    assert res["owner"] == "0"
    assert res["group"] == "6"
    assert res["major"] == 253
    assert res["minor"] == 10
    assert res["date"] == "Aug  4 16:56"
    assert res["name"] == "dm-10"
    assert res["dir"] == "/tmp"


def test_files_with_selinux_disabled():
    results = parse(FILES_CREATED_WITH_SELINUX_DISABLED.splitlines(), "/dev/mapper")
    assert len(results) == 1
    assert results["/dev/mapper"]["total"] == 2
    assert results["/dev/mapper"]["name"] == "/dev/mapper", results[0]["name"]
    res = results["/dev/mapper"]["entries"]["lv_cpwtk001_data01"]
    assert res["type"] == "l"
    assert res["links"] == 1
    assert res["owner"] == "0"
    assert res["group"] == "0"
    assert res["size"] == 7
    assert res["date"] == "Apr 27 05:34"
    assert res["name"] == "lv_cpwtk001_data01"
    assert res["link"] == "../dm-7"
    assert res["dir"] == "/dev/mapper"


def test_bad_line():
    results = parse(COMPLICATED_FILES_BAD_LINE.splitlines(), "/tmp")
    assert len(results) == 1
    assert results["/tmp"]["total"] == 16, results["/tmp"]["total"]
    assert results["/tmp"]["name"] == "/tmp", results["/tmp"]["name"]
    res = results["/tmp"]["entries"]["dm-10"]
    assert res["type"] == "b"
    assert res["links"] == 1
    assert res["owner"] == "0"
    assert res["group"] == "6"
    assert res["major"] == 253
    assert res["minor"] == 10
    assert res["date"] == "Aug  4 16:56"
    assert res["name"] == "dm-10"
    assert res["dir"] == "/tmp"


def test_rhel8_selinux():
    results = parse(RHEL8_SELINUX_DIRECTORY.splitlines(), "/var/lib/nova/instances")
    assert len(results) == 1
    assert results["/var/lib/nova/instances"]["name"] == "/var/lib/nova/instances", results["/var/lib/nova/instances"]["name"]
    res = results["/var/lib/nova/instances"]["entries"]["abcd-efgh-ijkl-mnop"]
    assert results["/var/lib/nova/instances"]["total"] == 0, results["/var/lib/nova/instances"]["total"]
    assert res["type"] == "d"
    assert res["links"] == 2
    assert res["owner"] == "root"
    assert res["group"] == "root"
    assert res["se_user"] == "unconfined_u"
    assert res["se_role"] == "object_r"
    assert res["se_type"] == "var_lib_t"
    assert res["se_mls"] == "s0"
    assert res["date"] == "Apr  8 16:41"
    assert res["name"] == "abcd-efgh-ijkl-mnop"
    assert res["dir"] == "/var/lib/nova/instances"



"""
CertList - command ``getcert list``
====================================

"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import keyword_search
from insights.specs import Specs


@parser(Specs.getcert_list)
class CertList(CommandParser):
    """
    Parse the output of ``getcert list``.

    Stores data as a pseudo-dictionary, keyed on request ID.  But it's much
    easier to find requests based on their properties, using the ``search``
    method.  This finds requests based on their keys, e.g.
    ``search(stuck='no')``.  Spaces and dashes are converted to underscores
    in the keys being sought, so one can search for ``key_pair_storage`` or
    ``pre_save_command``.  Multiple keys can be searched in the same call,
    e.. ``search(CA="IPA", stuck='yes')``.  If no keys are given, no requests
    are returned.

    Sample output::

        Number of certificates and requests being tracked: 2.
        Request ID '20130725003533':
                status: MONITORING
                stuck: no
                key pair storage: type=NSSDB,location='/etc/dirsrv/slapd-LDAP-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB',pinfile='/etc/dirsrv/slapd-LDAP-EXAMPLE-COM/pwdfile.txt'
                certificate: type=NSSDB,location='/etc/dirsrv/slapd-LDAP-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB'
                CA: IPA
                issuer: CN=Certificate Authority,O=LDAP.EXAMPLE.COM
                subject: CN=master.LDAP.EXAMPLE.COM,O=LDAP.EXAMPLE.COM
                expires: 2017-06-28 12:52:12 UTC
                eku: id-kp-serverAuth,id-kp-clientAuth
                pre-save command:
                post-save command: /usr/lib64/ipa/certmonger/restart_dirsrv LDAP-EXAMPLE-COM
                track: yes
                auto-renew: yes
        Request ID '20130725003602':
                status: MONITORING
                stuck: no
                key pair storage: type=NSSDB,location='/etc/dirsrv/slapd-PKI-IPA',nickname='Server-Cert',token='NSS Certificate DB',pinfile='/etc/dirsrv/slapd-PKI-IPA/pwdfile.txt'
                certificate: type=NSSDB,location='/etc/dirsrv/slapd-PKI-IPA',nickname='Server-Cert',token='NSS Certificate DB'
                CA: IPA
                issuer: CN=Certificate Authority,O=EXAMPLE.COM
                subject: CN=ldap.EXAMPLE.COM,O=EXAMPLE.COM
                expires: 2017-06-28 12:52:13 UTC
                eku: id-kp-serverAuth,id-kp-clientAuth
                pre-save command:
                post-save command: /usr/lib64/ipa/certmonger/restart_dirsrv PKI-IPA
                track: yes
                auto-renew: yes

    Attributes:

        num_tracked (int): The number of 'tracked' certificates and requests,
            as given in the first line of the output.
        requests (list): The list of request IDs as they appear in the output,
            as strings.

    Examples:

        >>> certs = shared[Cert_List]
        >>> certs.num_tracked  # number of certificates tracked from first line
        2
        >>> len(certs)  # number of requests stored - may be smaller than num_tracked
        2
        >>> certs.requests
        ['20130725003533', '20130725003602']
        >>> '20130725003533' in certs
        True
        >>> certs['20130725003533']['issuer']
        'CN=Certificate Authority,O=LDAP.EXAMPLE.COM'
        >>> for request in certs.search(CA='IPA'):
        ...     print request['certificate']
        ...
        type=NSSDB,location='/etc/dirsrv/slapd-LDAP-EXAMPLE-COM',nickname='Server-Cert',token='NSS Certificate DB'
        type=NSSDB,location='/etc/dirsrv/slapd-PKI-IPA',nickname='Server-Cert',token='NSS Certificate DB'

    """

    def parse_content(self, content):
        """
        We're only interested in lines that contain a ':'.  Special lines
        start with 'Request ID' and 'Number of certificates...'; we handle
        those separately.  All other lines are stripped of surrounding white
        space and stored as a key-value pair against the last request ID.
        """
        self._data = {}
        self.num_tracked = 0
        self.requests = []
        self._rq_list = []
        current_request = None

        _TRACK_HEADER = 'Number of certificates and requests being tracked: '
        _RQ_HEADER = 'Request ID '
        for line in content:
            line = line.strip()
            if line.startswith(_TRACK_HEADER):
                num_tracked = line[len(_TRACK_HEADER):-1]
                if not num_tracked.isdigit():
                    raise ParseException("Incorrectly formatted number of certificates and requests")
                self.num_tracked = int(num_tracked)
            elif line.startswith(_RQ_HEADER):
                current_request = line[len(_RQ_HEADER) + 1:-2]
                if current_request in self._data:
                    raise ParseException("Found duplicate request ID '{rq}'".format(rq=current_request))
                self._data[current_request] = {}
                self.requests.append(current_request)
                self._rq_list.append(self._data[current_request])
            elif line.endswith(':'):
                # Key with no value - fake it
                key = line[:-1]
                self._data[current_request][key] = ''
            elif ': ' in line:
                key, val = line.split(': ', 1)
                self._data[current_request][key] = val

    def __contains__(self, rq):
        """
        Does the certificate collection contain the given request ID?
        """
        return rq in self._data

    def __len__(self):
        """
        Return the number of requests found (not the number tracked)
        """
        return len(self._data)

    def __getitem__(self, rq):
        """
        Return the request with the given ID.
        """
        return self._data[rq]

    def search(self, **kwargs):
        """
        Search for one or more key-value pairs in the given data.  See the
        documentation of meth:insights.parsers.keyword_search for more
        details on how to use it.
        """
        return keyword_search(self._rq_list, **kwargs)



"""
Handle adding files and preparing the archive for upload
"""
from __future__ import absolute_import
from signal import SIGTERM, signal
import sys
import time
import os
import shutil
import subprocess
import shlex
import logging
import tempfile
import re
import atexit

from .utilities import determine_hostname, _expand_paths, write_data_to_file
from .insights_spec import InsightsFile, InsightsCommand
from .constants import InsightsConstants as constants

logger = logging.getLogger(__name__)


class InsightsArchive(object):
    """
    This class is an interface for adding command output
    and files to the insights archive

    Attributes:
        config          - an InsightsConfig object
        tmp_dir         - a temporary directory in /var/tmp
        archive_dir     - location to collect archive data inside tmp_dir
        archive_name    - filename of the archive and archive_dir
        cmd_dir         - insights_commands directory inside archive_dir
        compressor      - tar compression flag to use
        tar_file        - path of the final archive file
    """
    def __init__(self, config):
        """
        Initialize the Insights Archive
        """
        self.config = config
        # clean old temporary archive
        self.cleanup_previous_archive()

        # Create a random temp directory
        # input this to core collector as `tmp_path`
        self.tmp_dir = tempfile.mkdtemp(dir=constants.insights_tmp_path, prefix=constants.insights_tmp_prefix + '-')

        # We should not hint the hostname in the archive if it has to be obfuscated
        if config.obfuscate_hostname:
            hostname = "localhost"
        else:
            hostname = determine_hostname()

        self.archive_name = ("insights-%s-%s" %
                             (hostname,
                              time.strftime("%Y%m%d%H%M%S")))

        # lazy create these, only if needed when certain
        #   functions are called
        # classic collection and compliance needs these
        # core collection will set "archive_dir" on its own
        self.archive_dir = None
        self.cmd_dir = None

        self.compressor = config.compressor
        self.archive_stored = None
        self.tar_file = None
        self.keep_archive_dir = '/var/cache/insights-client'
        atexit.register(self.cleanup_tmp)
        signal(SIGTERM, self.sigterm_handler)

    def create_archive_dir(self):
        """
        Create the archive directory if it is undefined or does not exist.
        """
        if self.archive_dir and os.path.exists(self.archive_dir):
            # attr defined and exists. move along
            return self.archive_dir

        archive_dir = os.path.join(self.tmp_dir, self.archive_name)
        if not os.path.exists(archive_dir):
            logger.debug('Creating archive directory %s...', archive_dir)
            os.makedirs(archive_dir, 0o700)
        self.archive_dir = archive_dir
        return self.archive_dir

    def create_command_dir(self):
        """
        Create the "insights_commands" dir
        """
        self.create_archive_dir()
        cmd_dir = os.path.join(self.archive_dir, "insights_commands")
        logger.debug('Creating command directory %s...', cmd_dir)
        if not os.path.exists(cmd_dir):
            os.makedirs(cmd_dir, 0o700)
        self.cmd_dir = cmd_dir
        return self.cmd_dir

    def get_full_archive_path(self, path):
        """
        Returns the full archive path
        """
        self.create_archive_dir()
        return os.path.join(self.archive_dir, path.lstrip('/'))

    def _copy_file(self, path):
        """
        Copy just a single file
        """
        full_path = self.get_full_archive_path(path)
        # Try to make the dir, eat exception if it fails
        try:
            os.makedirs(os.path.dirname(full_path))
        except OSError:
            pass
        logger.debug("Copying %s to %s", path, full_path)
        shutil.copyfile(path, full_path)
        return path

    def copy_file(self, path):
        """
        Copy a single file or regex, creating the necessary directories
        """
        if "*" in path:
            paths = _expand_paths(path)
            if paths:
                for path in paths:
                    self._copy_file(path)
        else:
            if os.path.isfile(path):
                return self._copy_file(path)
            else:
                logger.debug("File %s does not exist", path)
                return False

    def copy_dir(self, path):
        """
        Recursively copy directory
        """
        self.create_archive_dir()
        for directory in path:
            if os.path.isdir(path):
                full_path = os.path.join(self.archive_dir, directory.lstrip('/'))
                logger.debug("Copying %s to %s", directory, full_path)
                shutil.copytree(directory, full_path)
            else:
                logger.debug("Not a directory: %s", directory)
        return path

    def get_compression_flag(self, compressor):
        return {
            "gz": "z",
            "xz": "J",
            "bz2": "j",
            "none": ""
        }.get(compressor, "z")

    def create_tar_file(self):
        """
        Create tar file to be compressed
        """
        if not self.tmp_dir:
            # we should never get here but bail out if we do
            raise RuntimeError('Archive temporary directory not defined.')
        tar_file_name = os.path.join(self.tmp_dir, self.archive_name)
        ext = "" if self.compressor == "none" else ".%s" % self.compressor
        tar_file_name = tar_file_name + ".tar" + ext
        logger.debug("Tar File: " + tar_file_name)
        # change cwd to compress the archive directory inside the temporary directory
        # with the correct structure
        return_code = subprocess.call(shlex.split("tar c%sfS %s %s" % (
            self.get_compression_flag(self.compressor),
            tar_file_name, self.archive_name)),
            stderr=subprocess.PIPE, cwd=self.tmp_dir)
        if (self.compressor in ["bz2", "xz"] and return_code != 0):
            logger.error("ERROR: %s compressor is not installed, cannot compress file", self.compressor)
            return None
        self.delete_archive_dir()
        logger.debug("Tar File Size: %s", str(os.path.getsize(tar_file_name)))
        self.tar_file = tar_file_name
        return tar_file_name

    def delete_tmp_dir(self):
        """
        Delete the entire tmp dir
        """
        if self.tmp_dir:
            logger.debug("Deleting: " + self.tmp_dir)
            shutil.rmtree(self.tmp_dir, True)

    def delete_archive_dir(self):
        """
        Delete the entire archive dir
        """
        if self.archive_dir:
            logger.debug("Deleting: " + self.archive_dir)
            shutil.rmtree(self.archive_dir, True)

    def add_to_archive(self, spec):
        '''
        Add files and commands to archive
        Use InsightsSpec.get_output() to get data
        '''
        ab_regex = [
            "^timeout: failed to run command .+: No such file or directory$",
            "^Missing Dependencies:"
        ]
        if isinstance(spec, InsightsCommand):
            archive_path = os.path.join(self.cmd_dir, spec.archive_path.lstrip('/'))
        if isinstance(spec, InsightsFile):
            archive_path = self.get_full_archive_path(spec.archive_path.lstrip('/'))
        output = spec.get_output()
        if output and not any(re.search(rg, output) for rg in ab_regex):
            write_data_to_file(output, archive_path)

    def add_metadata_to_archive(self, metadata, meta_path):
        '''
        Add metadata to archive
        '''
        archive_path = self.get_full_archive_path(meta_path.lstrip('/'))
        write_data_to_file(metadata, archive_path)

    def cleanup_tmp(self):
        '''
        DEPRECATE in favor of systemd-tmpfiles removing files.
        Only used during built-in collection.
        Delete archive and tmp dirs on exit unless --keep-archive is specified
            and tar_file exists.
        '''
        if self.config.keep_archive and self.tar_file:
            self.storing_archive()
            if self.config.no_upload:
                logger.info('Archive saved at %s', self.archive_stored)
            else:
                logger.info('Insights archive retained in %s', self.archive_stored)
        self.delete_tmp_dir()

    def sigterm_handler(_signo, _stack_frame, _context):
        sys.exit(1)

    def cleanup_previous_archive(self):
        '''
        Used at the start, this will clean the temporary directory of previous killed runs
        '''
        try:
            one_day_ago = time.time() - 24 * 60 * 60
            # list insights-client temporary directories
            directories = [os.path.join(constants.insights_tmp_path, directory)
                           for directory in os.listdir(constants.insights_tmp_path)
                           if directory.startswith(constants.insights_tmp_prefix)]
            # filter only those last modified more than 24 hours ago
            directories = [directory
                           for directory in directories
                           if os.path.isdir(directory) and os.path.getmtime(directory) <= one_day_ago]
            # remove the directories
            for directory in directories:
                logger.debug("Deleting previous archive %s", directory)
                shutil.rmtree(directory, True)
        except Exception:
            logger.error("ERROR: Could not remove old temporary directory")

    def storing_archive(self):
        if not os.path.exists(self.keep_archive_dir):
            try:
                os.makedirs(self.keep_archive_dir)
            except OSError:
                logger.error('ERROR: Could not create %s', self.keep_archive_dir)
                raise

        archive_name = os.path.basename(self.tar_file)
        self.archive_stored = os.path.join(self.keep_archive_dir, archive_name)
        logger.info('Copying archive from %s to %s', self.tar_file, self.archive_stored)
        try:
            shutil.copyfile(self.tar_file, self.archive_stored)
        except OSError:
            # file exists already
            logger.error('ERROR: Could not stored archive to %s', self.archive_stored)
            raise



"""
RndcStatus - Command ``rndc status``
=====================================
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.rndc_status)
class RndcStatus(CommandParser, dict):
    """
    Class for parsing the output of `rndc status` command.

    Typical output of the command is::

        version: BIND 9.11.4-P2-RedHat-9.11.4-9.P2.el7 (Extended Support Version) <id:7107deb>
        running on rhel7: Linux x86_64 3.10.0-957.10.1.el7.x86_64 #1 SMP Thu Feb 7 07:12:53 UTC 2019
        boot time: Mon, 26 Aug 2019 02:17:03 GMT
        last configured: Mon, 26 Aug 2019 02:17:03 GMT
        configuration file: /etc/named.conf
        CPUs found: 4
        worker threads: 4
        UDP listeners per interface: 3
        number of zones: 103 (97 automatic)
        debug level: 0
        xfers running: 0
        xfers deferred: 0
        soa queries in progress: 0
        query logging is OFF
        recursive clients: 0/900/1000
        tcp clients: 1/150
        server is up and running

    Raises:
        SkipComponent: When input is empty.
        ParseException: When input cannot be parsed.

    Examples:
        >>> type(rndc_status)
        <class 'insights.parsers.rndc_status.RndcStatus'>
        >>> rndc_status['CPUs found']
        '4'
        >>> rndc_status['server']
        'up and running'
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty content")
        for line in content:
            if ':' in line:
                k, v = line.split(':', 1)
            elif ' is ' in line:
                k, v = line.split(' is ', 1)
            else:
                raise ParseException("Incorrect content: '{0}'".format(content))
            self[k.strip()] = v.strip()



from insights.tests import context_wrap
from insights.parsers.sysconfig import GrubSysconfig

GRUB_SYSCONFIG = """

GRUB_TIMEOUT=1
GRUB_DISTRIBUTOR="$(sed 's, release .*$,,g' /etc/system-release)"
GRUB_DEFAULT=saved
GRUB_DISABLE_SUBMENU=true
GRUB_TERMINAL_OUTPUT="console"
GRUB_CMDLINE_LINUX="console=ttyS0 console=ttyS0,115200n8 no_timer_check net.ifnames=0 crashkernel=auto"
GRUB_DISABLE_RECOVERY="true"
GRUB_ENABLE_BLSCFG=true
""".strip()


def test_sysconfig_grub():
    result = GrubSysconfig(context_wrap(GRUB_SYSCONFIG))
    assert result["GRUB_ENABLE_BLSCFG"] == 'true'
    assert result.get("GRUB_ENABLE_BLSCFG") == 'true'
    assert result.get("NONEXISTENT_VAR") is None
    assert "NONEXISTENT_VAR" not in result
    assert "GRUB_ENABLE_BLSCFG" in result



import doctest
import pytest

from insights import SkipComponent
from insights.parsers import luksmeta
from insights.tests import context_wrap


LUKSMETA_OUTPUT = """0   active empty
1   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
2   active empty
3   active empty
4 inactive empty
5   active empty
6   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
7   active cb6e8904-81ff-40da-a84a-07ab9ab5715e
"""  # noqa

LUKSMETA_NOT_FOUND = "bash: luksmeta: command not found..."
LUKSMETA_NOT_INITIALIZED = "Device is not initialized (./luks1)"
LUKSMETA_BAD_DEVICE = "./luks2 (LUKS2) is not a LUKSv1 device"


def test_luksmeta():
    luksmeta_parsed = luksmeta.LuksMeta(context_wrap(LUKSMETA_OUTPUT, path="/insights_commands/cryptsetup_luksDump_--disable-external-tokens_.dev.disk.by-uuid.d62357eb-ea88-4b13-b756-a24e91fbfe9a"))

    with pytest.raises(SkipComponent):
        luksmeta.LuksMeta(context_wrap(LUKSMETA_NOT_FOUND))

    with pytest.raises(SkipComponent):
        luksmeta.LuksMeta(context_wrap(LUKSMETA_NOT_INITIALIZED))

    with pytest.raises(SkipComponent):
        luksmeta.LuksMeta(context_wrap(LUKSMETA_BAD_DEVICE))

    # 8 keyslots and 1  device UUID
    assert len(luksmeta_parsed) == 9
    assert "device_uuid" in luksmeta_parsed
    assert luksmeta_parsed["device_uuid"] == "d62357eb-ea88-4b13-b756-a24e91fbfe9a"

    for i in range(8):
        assert luksmeta_parsed[i].index == i

    assert str(luksmeta_parsed[0]) == "Keyslot on index 0 is 'active' with no embedded metadata"
    assert str(luksmeta_parsed[1]) == "Keyslot on index 1 is 'active' with metadata stored by application with UUID 'cb6e8904-81ff-40da-a84a-07ab9ab5715e'"

    assert luksmeta_parsed[0].state == "active"
    assert luksmeta_parsed[4].state == "inactive"

    assert luksmeta_parsed[0].metadata is None
    assert luksmeta_parsed[1].metadata is not None
    assert luksmeta_parsed[1].metadata == "cb6e8904-81ff-40da-a84a-07ab9ab5715e"

    luksmeta_parsed = luksmeta.LuksMeta(context_wrap(LUKSMETA_OUTPUT, path="/insights_commands/cryptsetup_luksDump_--disable-external-tokens_.dev.loop0"))
    assert "device_uuid" not in luksmeta_parsed


def test_doc_examples():
    env = {
            'parsed_result': luksmeta.LuksMeta(context_wrap(LUKSMETA_OUTPUT)),
          }
    failed, total = doctest.testmod(luksmeta, globs=env)
    assert failed == 0



from insights.parsers import openshift_configuration
from insights.tests import context_wrap

NODE_CONFIG = """
allowDisabledDocker: false
apiVersion: v1
dnsDomain: cluster.local
dnsIP: 10.66.208.145
dockerConfig:
  execHandlerName: ""
iptablesSyncPeriod: "30s"
imageConfig:
  format: openshift3/ose-${component}:${version}
  latest: false
kind: NodeConfig
kubeletArguments:
  node-labels: []
masterClientConnectionOverrides:
  acceptContentTypes: application/vnd.kubernetes.protobuf,application/json
  contentType: application/vnd.kubernetes.protobuf
  burst: 200
  qps: 100
masterKubeConfig: system:node:master.ose.com.kubeconfig
networkPluginName: redhat/openshift-ovs-subnet
# networkConfig struct introduced in origin 1.0.6 and OSE 3.0.2 which
# deprecates networkPluginName above. The two should match.
networkConfig:
   mtu: 1450
   networkPluginName: redhat/openshift-ovs-subnet
nodeName: master.ose.com
podManifestConfig:
servingInfo:
  bindAddress: 0.0.0.0:10250
  certFile: server.crt
  clientCA: ca.crt
  keyFile: server.key
volumeDirectory: /var/lib/origin/openshift.local.volumes
proxyArguments:
  proxy-mode:
     - iptables
volumeConfig:
  localQuota:
    perFSGroup:
"""

MASTER_CONFIG = """
admissionConfig:
apiLevels:
- v1
apiVersion: v1
assetConfig:
  logoutURL: ""
  masterPublicURL: https://master.ose.com:8443
  publicURL: https://master.ose.com:8443/console/
  servingInfo:
    bindAddress: 0.0.0.0:8443
    bindNetwork: tcp4
    certFile: master.server.crt
    clientCA: ""
    keyFile: master.server.key
    maxRequestsInFlight: 0
    requestTimeoutSeconds: 0
controllerConfig:
  serviceServingCert:
    signer:
      certFile: service-signer.crt
      keyFile: service-signer.key
controllers: '*'
corsAllowedOrigins:
  - 127.0.0.1
  - localhost
  - 10.66.208.145
  - kubernetes.default
  - kubernetes.default.svc.cluster.local
  - kubernetes
  - openshift.default
  - openshift.default.svc
  - 172.30.0.1
  - master.ose.com
  - openshift.default.svc.cluster.local
  - kubernetes.default.svc
  - openshift
dnsConfig:
  bindAddress: 0.0.0.0:8053
  bindNetwork: tcp4
etcdClientInfo:
  ca: master.etcd-ca.crt
  certFile: master.etcd-client.crt
  keyFile: master.etcd-client.key
  urls:
    - https://master.ose.com:2379
etcdStorageConfig:
  kubernetesStoragePrefix: kubernetes.io
  kubernetesStorageVersion: v1
  openShiftStoragePrefix: openshift.io
  openShiftStorageVersion: v1
imageConfig:
  format: openshift3/ose-${component}:${version}
  latest: false
kind: MasterConfig
kubeletClientInfo:
  ca: ca.crt
  certFile: master.kubelet-client.crt
  keyFile: master.kubelet-client.key
  port: 10250
kubernetesMasterConfig:
  apiServerArguments:
  controllerArguments:
  masterCount: 1
  masterIP: 10.66.208.145
  podEvictionTimeout:
  proxyClientInfo:
    certFile: master.proxy-client.crt
    keyFile: master.proxy-client.key
  schedulerArguments:
  schedulerConfigFile: /etc/origin/master/scheduler.json
  servicesNodePortRange: ""
  servicesSubnet: 172.30.0.0/16
  staticNodeNames: []
masterClients:
  externalKubernetesClientConnectionOverrides:
    acceptContentTypes: application/vnd.kubernetes.protobuf,application/json
    contentType: application/vnd.kubernetes.protobuf
    burst: 400
    qps: 200
  externalKubernetesKubeConfig: ""
  openshiftLoopbackClientConnectionOverrides:
    acceptContentTypes: application/vnd.kubernetes.protobuf,application/json
    contentType: application/vnd.kubernetes.protobuf
    burst: 600
    qps: 300
  openshiftLoopbackKubeConfig: openshift-master.kubeconfig
masterPublicURL: https://master.ose.com:8443
networkConfig:
  clusterNetworkCIDR: 10.128.0.0/14
  hostSubnetLength: 9
  networkPluginName: redhat/openshift-ovs-subnet
# serviceNetworkCIDR must match kubernetesMasterConfig.servicesSubnet
  serviceNetworkCIDR: 172.30.0.0/16
  externalIPNetworkCIDRs:
  - 0.0.0.0/0
oauthConfig:
  assetPublicURL: https://master.ose.com:8443/console/
  grantConfig:
    method: auto
  identityProviders:
  - challenge: true
    login: true
    mappingMethod: claim
    name: deny_all
    provider:
      apiVersion: v1
      kind: DenyAllPasswordIdentityProvider
  masterCA: ca-bundle.crt
  masterPublicURL: https://master.ose.com:8443
  masterURL: https://master.ose.com:8443
  sessionConfig:
    sessionMaxAgeSeconds: 3600
    sessionName: ssn
    sessionSecretsFile: /etc/origin/master/session-secrets.yaml
  tokenConfig:
    accessTokenMaxAgeSeconds: 86400
    authorizeTokenMaxAgeSeconds: 500
pauseControllers: false
policyConfig:
  bootstrapPolicyFile: /etc/origin/master/policy.json
  openshiftInfrastructureNamespace: openshift-infra
  openshiftSharedResourcesNamespace: openshift
projectConfig:
  defaultNodeSelector: ""
  projectRequestMessage: ""
  projectRequestTemplate: ""
  securityAllocator:
    mcsAllocatorRange: "s0:/2"
    mcsLabelsPerProject: 5
    uidAllocatorRange: "1000000000-1999999999/10000"
routingConfig:
  subdomain:  ""
serviceAccountConfig:
  limitSecretReferences: false
  managedNames:
  - default
  - builder
  - deployer
  masterCA: ca-bundle.crt
  privateKeyFile: serviceaccounts.private.key
  publicKeyFiles:
  - serviceaccounts.public.key
servingInfo:
  bindAddress: 0.0.0.0:8443
  bindNetwork: tcp4
  certFile: master.server.crt
  clientCA: ca.crt
  keyFile: master.server.key
  maxRequestsInFlight: 500
  requestTimeoutSeconds: 3600
volumeConfig:
  dynamicProvisioningEnabled: True
"""


def test_ose_node_config():
    result = openshift_configuration.OseMasterConfig(context_wrap(MASTER_CONFIG))
    assert result.data['assetConfig']['masterPublicURL'] == 'https://master.ose.com:8443'
    assert result.data['corsAllowedOrigins'][1] == 'localhost'
    assert result.data['projectConfig']['defaultNodeSelector'] == ""


def test_ose_master_config():
    result = openshift_configuration.OseNodeConfig(context_wrap(NODE_CONFIG))
    assert result.data['apiVersion'] == "v1"
    assert result.data['masterClientConnectionOverrides']['burst'] == 200



from insights.parsr.examples.httpd_conf import loads
from insights.parsr.query import ieq


DATA = r"""
<IfModule log_config_module>
    #
    # The following directives define some format nicknames for use with
    # a CustomLog directive (see below).
    #
    LogFormat "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\"" combined
    LogFormat "%h %l %u %t \"%r\" %>s %b" common
</IfModule>
    <IfModule logio_module>
      # You need to enable mod_logio.c to use %I and %O
      LogFormat "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\" %I %O" combinedio
    </IfModule>

    #
    # The location and format of the access logfile (Common Logfile Format).
    # If you do not define any access logfiles within a <VirtualHost>
    # container, they will be logged here.  Contrariwise, if you *do*
    # define per-<VirtualHost> access logfiles, transactions will be
    # logged therein and *not* in this file.
    #
    #CustomLog "logs/access_log" common
    CustomLog logs/ssl_request_log \
    "%t %h %{SSL_PROTOCOL}x %{SSL_CIPHER}x "%r" %b"

    #
    # If you prefer a logfile with access, agent, and referer information
    # (Combined Logfile Format) you can use the following directive.
    #
    CustomLog "logs/access_log" combined
"""

HTTPD_CONF_NEST_1 = """
<VirtualHost 128.39.140.28>
    <Directory /var/www/example>
        Options FollowSymLinks
        AllowOverride None
    </Directory>
    <IfModule mod_php4.c>
        php_admin_flag safe_mode Off
        php_admin_value register_globals    0
    </IfModule>
    DirectoryIndex index.php
    <IfModule mod_rewrite.c>
        RewriteEngine On
        RewriteRule .* /index.php
    </IfModule>
    <IfModule mod_rewrite.c>
        RewriteEngine Off
    </IfModule>
    <IfModule !php5_module>
        <IfModule !php4_module>
            <FilesMatch ".php[45]?$">
                Order allow,deny
                Deny from all
            </FilesMatch>
            <FilesMatch ".php[45]?$">
                Order deny,allow
            </FilesMatch>
        </IfModule>
    </IfModule>
    DocumentRoot /var/www/example
    ServerName www.example.com
    ServerAlias admin.example.com
</VirtualHost>
<IfModule !php5_module>
  <IfModule !php4_module>
    <Location />
        <FilesMatch ".php[45]">
            Order allow,deny
            Deny from all
        </FilesMatch>
    </Location>
  </IfModule>
</IfModule>
<IfModule mod_rewrite.c>
    RewriteEngine Off
</IfModule>
LogLevel warn
DocumentRoot "/var/www/html_cgi"
IncludeOptional conf.d/*.conf
EnableSendfile on
SSLProtocol -ALL +TLSv1.2  # SSLv3
""".strip()

HTTPD_CONF_NEST_2 = """
DocumentRoot "/var/www/html"
<VirtualHost 128.39.140.30>
    <IfModule !php5_module>
        <IfModule !php4_module>
            <FilesMatch ".php[45]?$">
                Order allow,deny
                Deny from all
            </FilesMatch>
            <FilesMatch ".php[45]?$">
                Order deny,allow
            </FilesMatch>
        </IfModule>
    </IfModule>
    DocumentRoot /var/www/example1
    ServerName www.example1.com
    ServerAlias admin.example1.com
</VirtualHost>
<IfModule !php5_module>
  <IfModule !php4_module>
    <Location />
        <FilesMatch test>
            Order deny,allow
            Allow from all
        </FilesMatch>
        <FilesMatch ".php[45]">
            Order deny,allow
        </FilesMatch>
    </Location>
  </IfModule>
</IfModule>
<IfModule mod_rewrite.c>
    RewriteEngine On
</IfModule>
EnableSendfile off
""".strip()

HTTPD_CONF_NEST_3 = """
<VirtualHost 128.39.140.28>
    <IfModule !php5_module>
        Testphp php5_v3_1
        <IfModule !php4_module>
            Testphp php4_v3_1
        </IfModule>
        Testphp php5_v3_2
    </IfModule>
</VirtualHost>
<IfModule !php5_module>
  Testphp php5_3_a
  <IfModule !php4_module>
    Testphp php4_3_a
  </IfModule>
</IfModule>
""".strip()

HTTPD_CONF_NEST_4 = """
<VirtualHost 128.39.140.30>
    <IfModule !php5_module>
        Testphp php5_v4_1
        <IfModule !php4_module>
            Testphp php4_v4_1
        </IfModule>
        Testphp php5_v4_2
    </IfModule>
</VirtualHost>
<IfModule !php5_module>
  Testphp php5_4_b
  <IfModule !php4_module>
    Testphp php4_4_b
  </IfModule>
</IfModule>
""".strip()


def test_if_module():
    val = loads(DATA)
    assert len(val) == 4


def test_httpd_conf_nest_one():
    val = loads(HTTPD_CONF_NEST_1)
    assert len(val["IfModule"]) == 2
    assert len(val["IfModule", "mod_rewrite.c"]) == 1
    assert len(val["IfModule", "!php5_module"]) == 1
    assert val["IfModule", "mod_rewrite.c"][0].lineno == 43
    assert val["LogLevel"].value == "warn"
    assert val["LogLevel"][0].lineno == 46
    assert val[ieq("loglevel")].value == "warn"
    assert val["SSLProtocol"][0].attrs == ["-ALL", "+TLSv1.2", "#", "SSLv3"]



"""
KpatchList - command ``/usr/sbin/kpatch list``
==============================================

The ``/usr/sbin/kpatch list`` command provides information about
the installed patch modules.

Sample content from command ``/usr/sbin/kpatch list`` is::
    Loaded patch modules:
    kpatch_3_10_0_1062_1_1_1_4 [enabled]

    Installed patch modules:
    kpatch_3_10_0_1062_1_1_1_4 (3.10.0-1062.1.1.el7.x86_64)

Examples:
    >>> 'kpatch_3_10_0_1062_1_1_1_4' in kpatchs.loaded
    True
    >>> kpatchs.loaded.get('kpatch_3_10_0_1062_1_1_1_4')
    'enabled'
    >>> 'kpatch_3_10_0_1062_1_1_1_4' in kpatchs.installed
    True
    >>> kpatchs.installed.get('kpatch_3_10_0_1062_1_1_1_4')
    '3.10.0-1062.1.1.el7.x86_64'
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.kpatch_list)
class KpatchList(CommandParser):
    """Class for command: /usr/sbin/kpatch list"""
    def parse_content(self, content):
        if not content:
            raise SkipComponent("No Data from command: /usr/sbin/kpatch list")

        self._loaded = {}
        self._installed = {}
        cur_dict = {}
        for line in content:
            if not line.strip():
                continue

            if 'Loaded patch modules' in line:
                cur_dict = self._loaded
                continue

            if 'Installed patch modules' in line:
                cur_dict = self._installed
                continue

            fields = [k.strip('()[]') for k in line.split()]
            if len(fields) == 1:
                if cur_dict == self._loaded:
                    # In the early version:
                    #  # kpatch list
                    #  Loaded patch modules:
                    #  kpatch_7_0_1_el7
                    #
                    cur_dict[fields[0].strip()] = ''

            elif len(fields) == 2:
                cur_dict[fields[0]] = fields[1]

    @property
    def loaded(self):
        """
        (dict): This will return the loaded kpath modules
        """
        return self._loaded

    @property
    def installed(self):
        """
        (dict): This will return the installed kpath modules
        """
        return self._installed



"""
Custom datasources for leapp
"""
import json
import os

from insights.core.context import HostContext
from insights.core.exceptions import ContentException, SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider


@datasource(HostContext)
def leapp_report(broker):
    """
    This datasource get useful information from ``leapp-report.json``.

    .. note::
        Since this file may contain sensitive information, this datasource does
        filter and only keep required data.

    Returns:
        str: JSON string after get the required data.

    Raises:
        SkipComponent: When the file does not exist or nothing need to collect
        ContentException: When any exception occurs.
    """
    valid_keys = {
        'inhibitor': ['title', 'summary', 'detail']
    }
    leapp_report_path = "/var/log/leapp/leapp-report.json"
    if not os.path.isfile(leapp_report_path):
        raise SkipComponent("No such file")
    try:
        with open(leapp_report_path, 'r') as fp:
            json_report = json.load(fp)
            if isinstance(json_report, dict):
                results = []
                for entry in json_report.get('entries', []):
                    groups = entry.get('groups', entry.get('flags', []))
                    for flag, keys in valid_keys.items():
                        if flag in groups:
                            ret = dict(groups=groups)
                            for key in keys:
                                detail = entry.get(key)
                                ret.update(detail if isinstance(detail, dict) else {key: detail}) if detail else None
                            results.append(ret)
                if results:
                    relative_path = 'insights_commands/leapp_report'
                    return DatasourceProvider(content=json.dumps(results), relative_path=relative_path)
                raise SkipComponent("Nothing")
    except Exception as e:
        raise ContentException(e)


@datasource(HostContext)
def migration_results(broker):
    """
    This datasource get useful information from ``/etc/migration-results``.

    .. note::
        Since this file may contain sensitive information, this datasource does
        filter and only keep required data.

    Returns:
        str: JSON string after get the required data.

    Raises:
        SkipComponent: When the file does not exist or nothing need to collect
        ContentException: When any exception occurs.
    """
    valid_keys = {
        'activity': [],
        'activity_ended': [],
        'activity_started': [],
        'env': [
            'LEAPP_CURRENT_PHASE',
            'LEAPP_EXPERIMENTAL',
            'LEAPP_NO_RHSM',
            'LEAPP_UNSUPPORTED'
        ],
        'run_id': [],
        'source_os': [],
        'success': [],
        'target_os': [],
        'version': [],
    }
    migration_results_file = "/etc/migration-results"
    if not os.path.isfile(migration_results_file):
        raise SkipComponent("No such file")
    try:
        with open(migration_results_file, 'r') as fp:
            json_report = json.load(fp)
            if isinstance(json_report, dict):
                results = []
                for activity in json_report.get('activities', []):
                    ret = {}
                    for key, sub_keys in valid_keys.items():
                        if key not in activity:
                            continue
                        val = activity.get(key)
                        if not sub_keys:
                            ret[key] = val
                        elif isinstance(val, dict):
                            sub_ret = {}
                            for sk in sub_keys:
                                sub_ret[sk] = (val or {}).get(sk)
                            if sub_ret:
                                ret[key] = sub_ret
                    results.append(ret) if ret else None
                if results:
                    relative_path = 'insights_commands/leapp_migration_results'
                    return DatasourceProvider(content=json.dumps(results), relative_path=relative_path)
                raise SkipComponent("Nothing")
    except Exception as e:
        raise ContentException(e)



"""
Custom datasources for user and group related specs

.. note::
    It should be noted that the specs in this module might contains sensitive
    information, please avoid collecting any of them.
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.specs import Specs


@datasource(HostContext)
def group_filters(broker):
    """
    Return a string contains the list of groups getting from the spec filter,
    separated with space.
    """
    grp_list = sorted(get_filters(Specs.group_info))
    if grp_list:
        return ' '.join(grp_list)
    raise SkipComponent



import doctest

from insights.parsers import ls_var_lib_pcp
from insights.parsers.ls_var_lib_pcp import LsVarLibPcp
from insights.tests import context_wrap

LS_VAR_LIB_PCP = """
total 16
drwxr-xr-x.  4 root root  33 Dec 15 08:12 .
drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..
drwxr-xr-x.  2 root root   6 Oct  3 09:37 pmcd
drwxr-xr-x.  2 root root   6 Oct  3 09:37 pmie
drwxrwxr-x.  4 root root 128 Aug 22 17:55 pmdas
"""


def test_ls_var_lib_pcp():
    ls_var_lib_pcp = LsVarLibPcp(context_wrap(LS_VAR_LIB_PCP, path="insights_commands/ls_-la_.var.lib.pcp"))
    assert ls_var_lib_pcp.dirs_of('/var/lib/pcp') == ['.', '..', 'pmcd', 'pmie', 'pmdas']
    journal = ls_var_lib_pcp.dir_entry('/var/lib/pcp', 'pmdas')
    assert journal is not None


def test_ls_var_lib_pcp_doc_examples():
    env = {
        'ls_var_lib_pcp': LsVarLibPcp(context_wrap(LS_VAR_LIB_PCP, path="insights_commands/ls_-la_.var.lib.pcp")),
    }
    failed, total = doctest.testmod(ls_var_lib_pcp, globs=env)
    assert failed == 0



"""
DuplicateMachine - file ``insights-commands/duplidate_machine_id``
==================================================================
"""

from insights.core.plugins import parser
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.specs import Specs


@parser(Specs.duplicate_machine_id)
class DuplicateMachine(Parser):
    """
    Parse the machine info in the file "insights-commands/duplidate_machine_info".

    Sample input::

        dc194312-8cdd-4e75-8cf1-2094bf666f45 hostname1,hostname2

    Attributes:
        duplicate_id (str): the duplicate machine id
        hostnames (list): A list of the hostnames with the same machine id

    Examples:
        >>> type(machine_id_obj)
        <class 'insights.parsers.machine_id.DuplicateMachine'>
        >>> machine_id_obj.duplicate_id
        'dc194312-8cdd-4e75-8cf1-2094bf666f45'
        >>> machine_id_obj.hostnames
        ['hostname1', 'hostname2']

    Raises:
        SkipComponent: when there is no expected content in the file
    """
    def parse_content(self, content):
        if len(content) == 1:
            machine_info = content[0].split(None, 1)
            if len(machine_info) == 2:
                self.duplicate_id = machine_info[0].strip()
                self.hostnames = machine_info[1].strip().split(',')
        if not hasattr(self, 'duplicate_id'):
            raise SkipComponent('No expected machine id found.')



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import max_uid
from insights.parsers.max_uid import MaxUID
from insights.tests import context_wrap


def test_max_uid():
    with pytest.raises(SkipComponent):
        MaxUID(context_wrap(""))

    with pytest.raises(ParseException):
        MaxUID(context_wrap("1a"))

    max_uid = MaxUID(context_wrap("65536"))
    assert max_uid is not None
    assert max_uid.value == 65536


def test_doc_examples():
    env = {
        'max_uid': MaxUID(context_wrap("65534")),
    }
    failed, total = doctest.testmod(max_uid, globs=env)
    assert failed == 0



"""
Nginx Configurations
====================

This module includes the following parsers:

NginxConfPEG - file ``/etc/nginx/nginx.conf`` and other Nginx configuration files
---------------------------------------------------------------------------------

ContainerNginxConfPEG - file ``/etc/nginx/nginx.conf`` and other Nginx configuration files of running containers
----------------------------------------------------------------------------------------------------------------
"""
import string

from insights.core import ConfigParser, ContainerParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsr import (EOF, EmptyQuotedString, Forward, LeftCurly, Lift, LineEnd, RightCurly,
                            Many, Number, OneLineComment, PosMarker, SemiColon, QuotedString,
                            skip_none, String, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section
from insights.specs import Specs


@parser(Specs.nginx_conf)
class NginxConfPEG(ConfigParser):
    """
    Parse the keyword-and-value of a Nginx configuration file.

    Example nginx.conf file::

        user       root
        worker_processes  5;
        error_log  logs/error.log;
        pid        logs/nginx.pid;
        worker_rlimit_nofile 8192;

        events {
          worker_connections  4096;
        }

        mail {
          server_name mail.example.com;
          auth_http  localhost:9000/cgi-bin/auth;
          server {
            listen   143;
            protocol imap;
          }
        }

        http {
          include  /etc/nginx/conf.d/*.conf
          index    index.html index.htm index.php;

          default_type application/octet-stream;
          log_format   main '$remote_addr - $remote_user [$time_local]  $status '
                            '"$request" $body_bytes_sent "$http_referer" '
                            '"$http_user_agent" "$http_x_forwarded_for"';
          access_log   logs/access.log  main;
          sendfile     on;
          tcp_nopush   on;
          server_names_hash_bucket_size 128;

          server { # php/fastcgi
            listen       80;
            server_name  domain1.com www.domain1.com;
            access_log   logs/domain1.access.log  main;
            root         html;

            location ~ \.php$ {
              fastcgi_pass   127.0.0.1:1025;
            }
          }

          server { # simple reverse-proxy
            listen       80;
            server_name  domain2.com www.domain2.com;
            access_log   logs/domain2.access.log  main;

            location ~ ^/(images|javascript|js|css|flash|media|static)/  {
              root    /var/www/virtual/big.server.com/htdocs;
              expires 30d;
            }

            location / {
              proxy_pass   http://127.0.0.1:8080;
            }
          }

          map $http_upgrade $connection_upgrade {
            default upgrade;
            '' close;
          }

          upstream websocket {
            server 10.66.208.205:8010;
          }

          upstream big_server_com {
            server 127.0.0.3:8000 weight=5;
            server 127.0.0.3:8001 weight=5;
            server 192.168.0.1:8000;
            server 192.168.0.1:8001;
          }

          server { # simple load balancing
            listen          80;
            server_name     big.server.com;
            access_log      logs/big.server.access.log main;

            location / {
              proxy_pass      http://big_server_com;
            }
          }
        }

    Examples:
        >>> type(nginxconfpeg)
        <class 'insights.parsers.nginx_conf.NginxConfPEG'>
        >>> nginxconfpeg['user'][-1].value
        'root'
        >>> nginxconfpeg['events']['worker_connections'][-1].value
        4096
        >>> nginxconfpeg['mail']['server'][0]['listen'][-1].value
        143
        >>> nginxconfpeg['http']['access_log'][-1].value
        'logs/access.log main'
        >>> nginxconfpeg['http']['server'][0]['location'][0]['fastcgi_pass'][-1].value
        '127.0.0.1:1025'
    """
    def __init__(self, *args, **kwargs):
        def to_entry(name, attrs, body):
            if body == ";":
                return Directive(name=name.value, attrs=attrs, lineno=name.lineno, src=self)
            return Section(name=name.value, attrs=attrs, children=body, lineno=name.lineno, src=self)

        name_chars = string.ascii_letters + "_/"
        Stmt = Forward()
        Num = Number & (WSChar | LineEnd | SemiColon)
        Comment = OneLineComment("#").map(lambda x: None)
        BeginBlock = WS >> LeftCurly << WS
        EndBlock = WS >> RightCurly << WS
        Bare = String(set(string.printable) - (set(string.whitespace) | set("#;{}'\"")))
        Name = WS >> PosMarker(String(name_chars) | EmptyQuotedString(name_chars)) << WS
        Attr = WS >> (Num | Bare | QuotedString) << WS
        Attrs = Many(Attr)
        Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
        Stanza = (Lift(to_entry) * Name * Attrs * (Block | SemiColon)) | Comment
        Stmt <= WS >> Stanza << WS
        Doc = Many(Stmt).map(skip_none)
        self.Top = Doc + EOF
        super(NginxConfPEG, self).__init__(*args, **kwargs)

    def parse_doc(self, content):
        try:
            return Entry(children=self.Top("\n".join(content))[0], src=self)
        except Exception:
            raise ParseException("There was an exception when parsing the config file.")


@parser(Specs.container_nginx_conf)
class ContainerNginxConfPEG(ContainerParser, NginxConfPEG):
    """
    Parser for the Nginx configuration files of running container.
    """
    pass



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import proc_environ
from insights.parsers.proc_environ import ProcEnviron, OpenshiftFluentdEnviron, OpenshiftRouterEnviron
from insights.tests import context_wrap

PROC_ENVIRON_EMPTY = """
""".strip()

PROC_ENVIRON_ERR = """
REGISTRIES
""".strip()

PROC_ENVIRON = """
REGISTRIES=--add-registry registry.access.redhat.com \x00OPTIONS= --selinux-enabled       --signature-verification=False\x00DOCKER_HTTP_HOST_COMPAT=1\x00ADD_REGISTRY=--add-registry registry.access.redhat.com\x00PATH=/usr/libexec/docker:/usr/bin:/usr/sbin\x00PWD=/run/docker/libcontainerd/containerd/135240dbd15a834acb21d68867930917afcc84c5f006ba65004acd88dccab756/init\x00LANG=en_US.UTF-8\x00GOTRACEBACK=crash\x00DOCKER_NETWORK_OPTIONS= --mtu=1450\x00DOCKER_CERT_PATH=/etc/docker\x00SHLVL=0\x00DOCKER_STORAGE_OPTIONS=--storage-driver devicemapper --storage-opt dm.fs=xfs --storage-opt dm.thinpooldev=/dev/mapper/docker--vg-docker--pool --storage-opt dm.use_deferred_removal=true --storage-opt dm.use_deferred_deletion=true \x00
""".strip()

OPENSHIFT_ROUTE_ENVIRON = """
PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin\x00HOSTNAME=node1\x00ROUTER_EXTERNAL_HOST_HOSTNAME=\x00ROUTER_METRICS_TLS_CERT_FILE=/etc/pki/tls/metrics/tls.crt\x00ROUTER_METRICS_TLS_KEY_FILE=/etc/pki/tls/metrics/tls.key\x00STATS_PASSWORD=9cBJqkmTE6\x00ROUTER_MAX_CONNECTIONS=25000\x00DEFAULT_CERTIFICATE_DIR=/etc/pki/tls/private\x00ROUTER_EXTERNAL_HOST_INSECURE=false\x00ROUTER_EXTERNAL_HOST_PASSWORD=\x00ROUTER_SERVICE_HTTP_PORT=80\x00ROUTER_CIPHERS=\x00ROUTER_EXTERNAL_HOST_INTERNAL_ADDRESS=\x00ROUTER_LISTEN_ADDR=0.0.0.0:1936\x00ROUTER_METRICS_TYPE=haproxy\x00ROUTER_SERVICE_NAME=router\x00ROUTER_EXTERNAL_HOST_USERNAME=\x00DEFAULT_CERTIFICATE_PATH=/etc/pki/tls/private/tls.crt\x00ROUTER_EXTERNAL_HOST_HTTPS_VSERVER=\x00ROUTER_EXTERNAL_HOST_PARTITION_PATH=\x00ROUTER_SERVICE_NAMESPACE=default\x00STATS_USERNAME=admin\x00ROUTER_EXTERNAL_HOST_PRIVKEY=/etc/secret-volume/router.pem\x00RELOAD_INTERVAL=10s\x00ROUTER_EXTERNAL_HOST_HTTP_VSERVER=\x00ROUTER_EXTERNAL_HOST_VXLAN_GW_CIDR=\x00ROUTER_SERVICE_HTTPS_PORT=443\x00ROUTER_SUBDOMAIN=\x00STATS_PORT=1936\x00ROUTER_SERVICE_PORT_443_TCP=443\x00ROUTER_PORT_80_TCP_PORT=80\x00KUBERNETES_PORT_443_TCP_PROTO=tcp\x00KUBERNETES_PORT_443_TCP_ADDR=172.30.0.1\x00KUBERNETES_PORT_53_UDP_PROTO=udp\x00KUBERNETES_PORT_53_UDP_PORT=53\x00REGISTRY_CONSOLE_SERVICE_HOST=172.30.207.124\x00REGISTRY_CONSOLE_PORT=tcp://172.30.207.124:9000\x00ROUTER_PORT_1936_TCP=tcp://172.30.106.114:1936\x00DOCKER_REGISTRY_SERVICE_HOST=172.30.43.64\x00DOCKER_REGISTRY_PORT=tcp://172.30.43.64:5000\x00KUBERNETES_PORT=tcp://172.30.0.1:443\x00ROUTER_PORT_80_TCP_ADDR=172.30.106.114\x00ROUTER_PORT_443_TCP_PORT=443\x00DOCKER_REGISTRY_SERVICE_PORT_5000_TCP=5000\x00DOCKER_REGISTRY_PORT_5000_TCP_ADDR=172.30.43.64\x00KUBERNETES_SERVICE_PORT=443\x00ROUTER_SERVICE_PORT_1936_TCP=1936\x00KUBERNETES_PORT_53_TCP=tcp://172.30.0.1:53\x00KUBERNETES_PORT_53_TCP_ADDR=172.30.0.1\x00KUBERNETES_SERVICE_PORT_HTTPS=443\x00KUBERNETES_SERVICE_PORT_DNS=53\x00REGISTRY_CONSOLE_PORT_9000_TCP_PROTO=tcp\x00REGISTRY_CONSOLE_PORT_9000_TCP_PORT=9000\x00ROUTER_PORT_80_TCP=tcp://172.30.106.114:80\x00ROUTER_PORT_443_TCP=tcp://172.30.106.114:443\x00ROUTER_PORT_443_TCP_PROTO=tcp\x00DOCKER_REGISTRY_PORT_5000_TCP_PROTO=tcp\x00KUBERNETES_SERVICE_PORT_DNS_TCP=53\x00KUBERNETES_PORT_443_TCP_PORT=443\x00KUBERNETES_PORT_53_UDP=udp://172.30.0.1:53\x00KUBERNETES_PORT_53_TCP_PORT=53\x00REGISTRY_CONSOLE_SERVICE_PORT_REGISTRY_CONSOLE=9000\x00REGISTRY_CONSOLE_PORT_9000_TCP_ADDR=172.30.207.124\x00ROUTER_PORT_80_TCP_PROTO=tcp\x00ROUTER_PORT_1936_TCP_PORT=1936\x00DOCKER_REGISTRY_PORT_5000_TCP=tcp://172.30.43.64:5000\x00KUBERNETES_PORT_53_UDP_ADDR=172.30.0.1\x00ROUTER_SERVICE_PORT_80_TCP=80\x00ROUTER_PORT_443_TCP_ADDR=172.30.106.114\x00DOCKER_REGISTRY_PORT_5000_TCP_PORT=5000\x00KUBERNETES_SERVICE_HOST=172.30.0.1\x00ROUTER_PORT_1936_TCP_ADDR=172.30.106.114\x00DOCKER_REGISTRY_SERVICE_PORT=5000\x00REGISTRY_CONSOLE_SERVICE_PORT=9000\x00REGISTRY_CONSOLE_PORT_9000_TCP=tcp://172.30.207.124:9000\x00ROUTER_SERVICE_HOST=172.30.106.114\x00ROUTER_SERVICE_PORT=80\x00ROUTER_PORT=tcp://172.30.106.114:80\x00ROUTER_PORT_1936_TCP_PROTO=tcp\x00KUBERNETES_PORT_443_TCP=tcp://172.30.0.1:443\x00KUBERNETES_PORT_53_TCP_PROTO=tcp\x00container=oci\x00HOME=/root\x00KUBECONFIG=/var/lib/origin/openshift.local.config/master/admin.kubeconfig\x00TEMPLATE_FILE=/var/lib/haproxy/conf/haproxy-config.template\x00RELOAD_SCRIPT=/var/lib/haproxy/reload-haproxy\x00
"""

OPENSHIFT_FLUENTD_ENVIRON = """
PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin\x00container=oci\x00LOGGING_FILE_SIZE=1024000\x00LOGGING_FILE_AGE=10\x00LOGGING_FILE_PATH=/var/log/fluentd/fluentd.log\x00
"""


def test_proc_environ():
    results = ProcEnviron(context_wrap(PROC_ENVIRON))
    assert results['REGISTRIES'] == '--add-registry registry.access.redhat.com'
    assert 'OPTIONS' in results


def test_invalid():
    with pytest.raises(ParseException) as e:
        ProcEnviron(context_wrap(PROC_ENVIRON_ERR))
    assert "Incorrect" in str(e)


def test_empty():
    with pytest.raises(SkipComponent) as e:
        ProcEnviron(context_wrap(PROC_ENVIRON_EMPTY))
    assert "Empty output." in str(e)


def test_proc_environ_doc_examples():
    env = {
        'proc_environ': ProcEnviron(
            context_wrap(PROC_ENVIRON)),
    }
    failed, total = doctest.testmod(proc_environ, globs=env)
    assert failed == 0


def test_openshift_router_proc_environ():
    results = OpenshiftRouterEnviron(context_wrap(OPENSHIFT_ROUTE_ENVIRON))
    assert results['RELOAD_INTERVAL'] == '10s'


def test_openshift_fluentd_proc_environ():
    results = OpenshiftFluentdEnviron(context_wrap(OPENSHIFT_FLUENTD_ENVIRON))
    assert results['LOGGING_FILE_SIZE'] == '1024000'



import yaml

from insights.core import Parser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.metadata_json)
class MetadataJson(Parser):
    def parse_content(self, content):
        self.data = yaml.safe_load(content)



"""
Rules for data collection
"""
from __future__ import absolute_import
import hashlib
import json
import logging
import six
import shlex
import os
import requests
import yaml
import stat
from six.moves import configparser as ConfigParser

from subprocess import Popen, PIPE, STDOUT
from tempfile import NamedTemporaryFile
from .constants import InsightsConstants as constants

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)
NETWORK = constants.custom_network_log_level


def correct_format(parsed_data, expected_keys, filename):
    '''
    Ensure the parsed file matches the needed format
    Returns True, <message> on error
    Returns False, None on success
    '''
    # validate keys are what we expect
    def is_list_of_strings(data):
        '''
        Helper function for correct_format()
        '''
        if data is None:
            # nonetype, no data to parse. treat as empty list
            return True
        if not isinstance(data, list):
            return False
        for l in data:
            if not isinstance(l, six.string_types):
                return False
        return True

    keys = parsed_data.keys()
    invalid_keys = set(keys).difference(expected_keys)
    if invalid_keys:
        return True, ('Unknown section(s) in %s: ' % filename + ', '.join(invalid_keys) +
                      '\nValid sections are ' + ', '.join(expected_keys) + '.')

    # validate format (lists of strings)
    for k in expected_keys:
        if k in parsed_data:
            if k == 'patterns' and isinstance(parsed_data['patterns'], dict):
                if 'regex' not in parsed_data['patterns']:
                    return True, 'Patterns section contains an object but the "regex" key was not specified.'
                if 'regex' in parsed_data['patterns'] and len(parsed_data['patterns']) > 1:
                    return True, 'Unknown keys in the patterns section. Only "regex" is valid.'
                if not is_list_of_strings(parsed_data['patterns']['regex']):
                    return True, 'regex section under patterns must be a list of strings.'
                continue
            if not is_list_of_strings(parsed_data[k]):
                return True, '%s section must be a list of strings.' % k
    return False, None


def load_yaml(filename):
    try:
        with open(filename) as f:
            loaded_yaml = yaml.safe_load(f)
        if loaded_yaml is None:
            logger.debug('%s is empty.', filename)
            return {}
    except (yaml.YAMLError, yaml.parser.ParserError) as e:
        # can't parse yaml from conf
        raise RuntimeError('ERROR: Cannot parse %s.\n'
                           'If using any YAML tokens such as [] in an expression, '
                           'be sure to wrap the expression in quotation marks.\n\nError details:\n%s\n' % (filename, e))
    if not isinstance(loaded_yaml, dict):
        # loaded data should be a dict with at least one key
        raise RuntimeError('ERROR: Invalid YAML loaded.')
    return loaded_yaml


def verify_permissions(f):
    '''
    Verify 600 permissions on a file
    '''
    mode = stat.S_IMODE(os.stat(f).st_mode)
    if not mode == 0o600:
        raise RuntimeError("Invalid permissions on %s. "
                           "Expected 0600 got %s" % (f, oct(mode)))
    logger.debug("Correct file permissions on %s", f)


class InsightsUploadConf(object):
    """
    Insights spec configuration from uploader.json
    """

    def __init__(self, config, conn=None):
        """
        Load config from parent
        """
        self.config = config
        self.fallback_file = constants.collection_fallback_file
        self.remove_file = config.remove_file
        self.redaction_file = config.redaction_file
        self.content_redaction_file = config.content_redaction_file
        self.tags_file = config.tags_file
        self.collection_rules_file = constants.collection_rules_file
        self.collection_rules_url = self.config.collection_rules_url
        self.gpg = self.config.gpg

        # initialize an attribute to store the content of uploader.json
        #   once it is loaded and verified
        self.uploader_json = None

        # set rm_conf as a class attribute so we can observe it
        #   in create_report
        self.rm_conf = None

        # attribute to set when using file-redaction.yaml instead of
        #   remove.conf, for reporting purposes. True by default
        #   since new format is favored.
        self.using_new_format = True

        if conn:
            if self.collection_rules_url is None:
                if config.legacy_upload:
                    self.collection_rules_url = conn.base_url + '/v1/static/uploader.v2.json'
                else:
                    self.collection_rules_url = conn.base_url.split('/platform')[0] + '/v1/static/uploader.v2.json'
            self.conn = conn

    def validate_gpg_sig(self, path, sig=None):
        """
        Validate the collection rules
        """
        logger.debug("Verifying GPG signature of Insights configuration")
        if sig is None:
            sig = path + ".asc"
        command = ("/usr/bin/gpg --no-default-keyring "
                   "--keyring " + constants.pub_gpg_path +
                   " --verify " + sig + " " + path)
        if not six.PY3:
            command = command.encode('utf-8', 'ignore')
        args = shlex.split(command)
        logger.debug("Executing: %s", args)
        proc = Popen(
            args, shell=False, stdout=PIPE, stderr=STDOUT, close_fds=True)
        stdout, stderr = proc.communicate()
        logger.debug("STDOUT: %s", stdout)
        logger.debug("STDERR: %s", stderr)
        logger.debug("Status: %s", proc.returncode)
        if proc.returncode:
            logger.error("ERROR: Unable to validate GPG signature: %s", path)
            return False
        else:
            logger.debug("GPG signature verified")
            return True

    def try_disk(self, path, gpg=True):
        """
        Try to load json off disk
        """
        if not os.path.isfile(path):
            return

        if not gpg or self.validate_gpg_sig(path):
            stream = open(path, 'r')
            json_stream = stream.read()
            if len(json_stream):
                try:
                    json_config = json.loads(json_stream)
                    return json_config
                except ValueError:
                    logger.error("ERROR: Invalid JSON in %s", path)
                    return False
            else:
                logger.warn("WARNING: %s was an empty file", path)
                return

    def get_collection_rules(self, raw=False):
        """
        Download the collection rules
        """
        logger.debug("Attemping to download collection rules from %s",
                     self.collection_rules_url)

        try:
            req = self.conn.get(
                self.collection_rules_url, headers=({'accept': 'text/plain'}))

            if req.status_code == 200:
                logger.debug("Successfully downloaded collection rules")

                json_response = NamedTemporaryFile()
                json_response.write(req.text.encode('utf-8'))
                json_response.file.flush()
            else:
                logger.error("ERROR: Could not download dynamic configuration")
                logger.error("Debug Info: \nConf status: %s", req.status_code)
                logger.error("Debug Info: \nConf message: %s", req.text)
                return None
        except requests.ConnectionError as e:
            logger.error(
                "ERROR: Could not download dynamic configuration: %s", e)
            return None

        if self.gpg:
            self.get_collection_rules_gpg(json_response)

        self.write_collection_data(self.collection_rules_file, req.text)

        if raw:
            return req.text
        else:
            return json.loads(req.text)

    def fetch_gpg(self):
        logger.debug("Attemping to download collection "
                     "rules GPG signature from %s",
                     self.collection_rules_url + ".asc")

        headers = ({'accept': 'text/plain'})
        config_sig = self.conn.get(self.collection_rules_url + '.asc',
                                   headers=headers)
        if config_sig.status_code == 200:
            logger.debug("Successfully downloaded GPG signature")
            return config_sig.text
        else:
            logger.error("ERROR: Download of GPG Signature failed!")
            logger.error("Sig status: %s", config_sig.status_code)
            return False

    def get_collection_rules_gpg(self, collection_rules):
        """
        Download the collection rules gpg signature
        """
        sig_text = self.fetch_gpg()
        sig_response = NamedTemporaryFile(suffix=".asc")
        sig_response.write(sig_text.encode('utf-8'))
        sig_response.file.flush()
        self.validate_gpg_sig(collection_rules.name, sig_response.name)
        self.write_collection_data(self.collection_rules_file + ".asc", sig_text)

    def write_collection_data(self, path, data):
        """
        Write collections rules to disk
        """
        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
        fd = os.open(path, flags, 0o600)
        with os.fdopen(fd, 'w') as dyn_conf_file:
            dyn_conf_file.write(data)

    def get_conf_file(self):
        """
        Get config from local config file, first try cache, then fallback.
        """
        if self.uploader_json:
            return self.uploader_json

        for conf_file in [self.collection_rules_file, self.fallback_file]:
            logger.debug("trying to read conf from: " + conf_file)
            conf = self.try_disk(conf_file, self.gpg)

            if not conf:
                continue

            version = conf.get('version', None)
            if version is None:
                raise ValueError("ERROR: Could not find version in json")

            conf['file'] = conf_file
            logger.debug("Success reading config")
            logger.debug(json.dumps(conf))
            self.uploader_json = conf
            return conf

        raise RuntimeError("ERROR: Unable to download conf or read it from disk!")

    def get_conf_update(self):
        """
        Get updated config from URL.
        """
        dyn_conf = self.get_collection_rules()

        if not dyn_conf:
            return self.get_conf_file()

        version = dyn_conf.get('version', None)
        if version is None:
            raise ValueError("ERROR: Could not find version in json")

        dyn_conf['file'] = self.collection_rules_file
        logger.debug("Success reading config")
        config_hash = hashlib.sha1(json.dumps(dyn_conf).encode('utf-8')).hexdigest()
        logger.debug('sha1 of config: %s', config_hash)
        return dyn_conf

    def get_rm_conf_old(self):
        """
        Get excluded files config from remove_file.
        """
        # Convert config object into dict
        self.using_new_format = False
        parsedconfig = ConfigParser.RawConfigParser()
        if not self.remove_file:
            # no filename defined, return nothing
            logger.debug('remove_file is undefined')
            return None
        if not os.path.isfile(self.remove_file):
            logger.debug('%s not found. No data files, commands,'
                         ' or patterns will be ignored, and no keyword obfuscation will occur.', self.remove_file)
            return None
        try:
            verify_permissions(self.remove_file)
        except RuntimeError as e:
            if self.config.validate:
                # exit if permissions invalid and using --validate
                raise RuntimeError('ERROR: %s' % e)
            logger.warning('WARNING: %s', e)
        try:
            parsedconfig.read(self.remove_file)
            sections = parsedconfig.sections()

            if not sections:
                # file has no sections, skip it
                logger.debug('Remove.conf exists but no parameters have been defined.')
                return None

            if sections != ['remove']:
                raise RuntimeError('ERROR: invalid section(s) in remove.conf. Only "remove" is valid.')

            expected_keys = ('commands', 'files', 'patterns', 'keywords')
            rm_conf = {}
            for item, value in parsedconfig.items('remove'):
                if item not in expected_keys:
                    raise RuntimeError('ERROR: Unknown key in remove.conf: ' + item +
                                       '\nValid keys are ' + ', '.join(expected_keys) + '.')
                if six.PY3:
                    rm_conf[item] = [v.strip() for v in value.strip().encode('utf-8').decode('unicode-escape').split(',')]
                else:
                    rm_conf[item] = [v.strip() for v in value.strip().decode('string-escape').split(',')]
            self.rm_conf = rm_conf
        except ConfigParser.Error as e:
            # can't parse config file at all
            logger.debug(e)
            logger.debug('To configure using YAML, please use file-redaction.yaml and file-content-redaction.yaml.')
            raise RuntimeError('ERROR: Cannot parse the remove.conf file.\n'
                               'See %s for more information.' % self.config.logging_file)
        logger.warning('WARNING: remove.conf is deprecated. Please use file-redaction.yaml and file-content-redaction.yaml. See https://access.redhat.com/articles/4511681 for details.')
        return self.rm_conf

    def load_redaction_file(self, fname):
        '''
        Load the YAML-style file-redaction.yaml
            or file-content-redaction.yaml files
        '''
        if fname not in (self.redaction_file, self.content_redaction_file):
            # invalid function use, should never get here in a production situation
            return None
        if not fname:
            # no filename defined, return nothing
            logger.debug('redaction_file or content_redaction_file is undefined')
            return None
        if not fname or not os.path.isfile(fname):
            if fname == self.redaction_file:
                logger.debug('%s not found. No files or commands will be skipped.', self.redaction_file)
            elif fname == self.content_redaction_file:
                logger.debug('%s not found. '
                             'No patterns will be skipped and no keyword obfuscation will occur.', self.content_redaction_file)
            return None
        try:
            verify_permissions(fname)
        except RuntimeError as e:
            if self.config.validate:
                # exit if permissions invalid and using --validate
                raise RuntimeError('ERROR: %s' % e)
            logger.warning('WARNING: %s', e)
        loaded = load_yaml(fname)
        if fname == self.redaction_file:
            err, msg = correct_format(loaded, ('commands', 'files', 'components'), fname)
        elif fname == self.content_redaction_file:
            err, msg = correct_format(loaded, ('patterns', 'keywords'), fname)
        if err:
            # YAML is correct but doesn't match the format we need
            raise RuntimeError('ERROR: ' + msg)
        return loaded

    def get_rm_conf(self):
        '''
        Try to load the the "new" version of
        remove.conf (file-redaction.yaml and file-redaction.yaml)
        '''
        rm_conf = {}
        redact_conf = self.load_redaction_file(self.redaction_file)
        content_redact_conf = self.load_redaction_file(self.content_redaction_file)

        if not redact_conf and not content_redact_conf:
            # no file-redaction.yaml or file-content-redaction.yaml defined,
            #   try to use remove.conf
            self.rm_conf = self.get_rm_conf_old()
        else:
            rm_conf.update(redact_conf or {})
            rm_conf.update(content_redact_conf or {})
            # remove Nones, empty strings, and empty lists
            self.rm_conf = dict((k, v) for k, v in rm_conf.items() if v)

        # return the RAW rm_conf
        return self.rm_conf

    def get_tags_conf(self):
        '''
        Try to load the tags.conf file
        '''
        if not os.path.isfile(self.tags_file):
            logger.info("%s does not exist", self.tags_file)
            return None
        else:
            try:
                load_yaml(self.tags_file)
                logger.info("%s loaded successfully", self.tags_file)
            except RuntimeError:
                logger.warning("Invalid YAML. Unable to load %s", self.tags_file)
                return None

    def validate(self):
        '''
        Validate remove.conf and tags.conf
        '''
        self.get_tags_conf()
        success = self.get_rm_conf()
        if not success:
            logger.info('No contents in the blacklist configuration to validate.')
            return None
        # Using print here as this could contain sensitive information
        print('Blacklist configuration parsed contents:')
        print(json.dumps(success, indent=4))
        logger.info('Parsed successfully.')
        return True

    def create_report(self):
        def length(lst):
            '''
            Because of how the INI remove.conf is parsed,
            an empty value in the conf will produce
            the value [''] when parsed. Do not include
            these in the report
            '''
            if len(lst) == 1 and lst[0] == '':
                return 0
            return len(lst)

        num_commands = 0
        num_files = 0
        num_components = 0
        num_patterns = 0
        num_keywords = 0
        using_regex = False

        if self.rm_conf:
            for key in self.rm_conf:
                if key == 'commands':
                    num_commands = length(self.rm_conf['commands'])
                if key == 'files':
                    num_files = length(self.rm_conf['files'])
                if key == 'components':
                    num_components = length(self.rm_conf['components'])
                if key == 'patterns':
                    if isinstance(self.rm_conf['patterns'], dict):
                        num_patterns = length(self.rm_conf['patterns']['regex'])
                        using_regex = True
                    else:
                        num_patterns = length(self.rm_conf['patterns'])
                if key == 'keywords':
                    num_keywords = length(self.rm_conf['keywords'])

        return {
            'obfuscate': self.config.obfuscate,
            'obfuscate_hostname': self.config.obfuscate_hostname,
            'commands': num_commands,
            'files': num_files,
            'components': num_components,
            'patterns': num_patterns,
            'keywords': num_keywords,
            'using_new_format': self.using_new_format,
            'using_patterns_regex': using_regex
        }



from insights.parsers import squid
from insights.tests import context_wrap

SQUID_CACHE_LOG = """
2023/11/01 06:00:33 kid1|   Took 0.00 seconds (  0.00 entries/sec).
FATAL: Couldn't start logfile helper
Squid Cache (Version 3.5.20): Terminated abnormally.
CPU Usage: 0.046 seconds = 0.033 user + 0.013 sys
Maximum Resident Size: 34784 KB
Page faults with physical i/o: 0
2023/11/01 06:01:16 kid1| Set Current Directory to /var/spool/squid
2023/11/01 06:01:16 kid1| Starting Squid Cache version 3.5.20 for x86_64-redhat-linux-gnu...
2023/11/01 06:01:16 kid1| Service Name: squid
""".strip()


def test_doc():
    """
    To make the examples readable, it's better to show one of the main usage "last_scan".
    And the last_scan should be called before the parser initialation.
    However, the initialization is done here, so the "last_scan" in the example won't work.
    As a result, it will rasise error when refering the result key.
    So we will not thest the examples, just show the users how to use it.
    """
    pass


def test_squid_cache_log():
    squid.SquidCacheLog.last_scan('start_error_test', 'Terminated abnormally')
    cache_log = squid.SquidCacheLog(context_wrap(SQUID_CACHE_LOG))
    assert cache_log.start_error_test
    assert cache_log.start_error_test.get('raw_message') == "Squid Cache (Version 3.5.20): Terminated abnormally."



"""
nmcli_dev_show command
======================
As there are three different file paths in different sos packages,
create this combiner to fix this issue.
"""
from insights.core.exceptions import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.nmcli import NmcliDevShow, NmcliDevShowSos


@combiner([NmcliDevShow, NmcliDevShowSos])
class AllNmcliDevShow(dict):
    """
    Combiner to combine return values from parser NmcliDevShow into one dict

    Examples:
        >>> allnmclidevshow['eth0']['TYPE']
        'ethernet'
        >>> allnmclidevshow.connected_devices
        ['eth0']
    """

    def __init__(self, nmclidevshow, nmclidevshowsos):
        data = {}
        self._con_dev = []

        if nmclidevshow:
            data.update(nmclidevshow)
            self._con_dev = nmclidevshow.connected_devices
        elif nmclidevshowsos:
            for item in nmclidevshowsos:
                data.update(item)
                self._con_dev.extend(item.connected_devices)

        if not data:
            raise SkipComponent()

        super(AllNmcliDevShow, self).__init__()
        self.update(data)

    @property
    def connected_devices(self):
        """(list): The list of devices who's state is connected and managed by NetworkManager"""
        return self._con_dev

    @property
    def data(self):
        """(dict): Dict with the device name as the key and NmcliDevShow details as the value."""
        return self
